@@ -479,6 +479,12 @@ const QOperatingSystemVersionBase QOperatingSystemVersion::Windows11_22H2;
     \since 6.6
  */
 
+/*!
+    \variable QOperatingSystemVersion::Windows11_24H2
+    \brief a version corresponding to Windows 11 Version 24H2 (version 10.0.26100).
+    \since 6.8.1
+ */
+
 /*!
     \variable QOperatingSystemVersion::OSXMavericks
     \brief a version corresponding to OS X Mavericks (version 10.9).
