@@ -928,7 +928,7 @@ QTextStream::QTextStream(QString *string, OpenMode openMode)
 {
 #if defined (QTEXTSTREAM_DEBUG)
     qDebug("QTextStream::QTextStream(QString *string == *%p, openMode = %d)",
-           string, int(openMode));
+           string, int(openMode.toInt()));
 #endif
     Q_D(QTextStream);
     d->string = string;
@@ -946,7 +946,7 @@ QTextStream::QTextStream(QByteArray *array, OpenMode openMode)
 {
 #if defined (QTEXTSTREAM_DEBUG)
     qDebug("QTextStream::QTextStream(QByteArray *array == *%p, openMode = %d)",
-           array, int(openMode));
+           array, int(openMode.toInt()));
 #endif
     Q_D(QTextStream);
     d->device = new QBuffer(array);
@@ -973,7 +973,7 @@ QTextStream::QTextStream(const QByteArray &array, OpenMode openMode)
 {
 #if defined (QTEXTSTREAM_DEBUG)
     qDebug("QTextStream::QTextStream(const QByteArray &array == *(%p), openMode = %d)",
-           &array, int(openMode));
+           &array, int(openMode.toInt()));
 #endif
     QBuffer *buffer = new QBuffer;
     buffer->setData(array);
@@ -1004,7 +1004,7 @@ QTextStream::QTextStream(FILE *fileHandle, OpenMode openMode)
 {
 #if defined (QTEXTSTREAM_DEBUG)
     qDebug("QTextStream::QTextStream(FILE *fileHandle = %p, openMode = %d)",
-           fileHandle, int(openMode));
+           fileHandle, int(openMode.toInt()));
 #endif
     QFile *file = new QFile;
     // Discarding the return value of open; even if it failed
@@ -1569,7 +1569,7 @@ QString QTextStream::readLine(qint64 maxlen)
     an error has occurred; otherwise returns \c true. The contents in
     \a line before the call are discarded in any case.
 
-    \sa readAll(), QIODevice::readLine()
+    \sa readAll(), QIODevice::readLine(), QIODevice::readLineInto()
 */
 bool QTextStream::readLineInto(QString *line, qint64 maxlen)
 {
@@ -1672,10 +1672,10 @@ QTextStreamPrivate::NumberParsingStatus QTextStreamPrivate::getNumber(qulonglong
         // Parse digits
         int ndigits = 0;
         while (getChar(&dig)) {
-            int n = dig.toLower().unicode();
-            if (n == '0' || n == '1') {
+            char16_t n = dig.toLower().unicode();
+            if (n == u'0' || n == u'1') {
                 val <<= 1;
-                val += n - '0';
+                val += n - u'0';
             } else {
                 ungetChar(dig);
                 break;
@@ -1692,16 +1692,16 @@ QTextStreamPrivate::NumberParsingStatus QTextStreamPrivate::getNumber(qulonglong
     }
     case 8: {
         QChar pf, dig;
-        // Parse prefix '0'
+        // Parse prefix u'0'
         if (!getChar(&pf) || pf != u'0')
             return npsInvalidPrefix;
         // Parse digits
         int ndigits = 0;
         while (getChar(&dig)) {
-            int n = dig.toLower().unicode();
+            char16_t n = dig.toLower().unicode();
             if (isOctalDigit(n)) {
                 val *= 8;
-                val += n - '0';
+                val += n - u'0';
             } else {
                 ungetChar(dig);
                 break;
@@ -1856,23 +1856,23 @@ bool QTextStreamPrivate::getReal(double *f)
     QChar c;
     while (getChar(&c)) {
         switch (c.unicode()) {
-        case '0': case '1': case '2': case '3': case '4':
-        case '5': case '6': case '7': case '8': case '9':
+        case u'0': case u'1': case u'2': case u'3': case u'4':
+        case u'5': case u'6': case u'7': case u'8': case u'9':
             input = InputDigit;
             break;
-        case 'i': case 'I':
+        case u'i': case u'I':
             input = InputI;
             break;
-        case 'n': case 'N':
+        case u'n': case u'N':
             input = InputN;
             break;
-        case 'f': case 'F':
+        case u'f': case u'F':
             input = InputF;
             break;
-        case 'a': case 'A':
+        case u'a': case u'A':
             input = InputA;
             break;
-        case 't': case 'T':
+        case u't': case u'T':
             input = InputT;
             break;
         default: {
@@ -2204,44 +2204,34 @@ QTextStream &QTextStream::operator>>(char *c)
  */
 void QTextStreamPrivate::putNumber(qulonglong number, bool negative)
 {
-    QString result;
-
     unsigned flags = 0;
     const QTextStream::NumberFlags numberFlags = params.numberFlags;
     if (numberFlags & QTextStream::ShowBase)
         flags |= QLocaleData::ShowBase;
-    if (numberFlags & QTextStream::ForceSign)
+    // ForceSign is irrelevant when we'll be including a sign in any case:
+    if ((numberFlags & QTextStream::ForceSign) && !negative)
         flags |= QLocaleData::AlwaysShowSign;
     if (numberFlags & QTextStream::UppercaseBase)
         flags |= QLocaleData::UppercaseBase;
     if (numberFlags & QTextStream::UppercaseDigits)
         flags |= QLocaleData::CapitalEorX;
 
-    // add thousands group separators. For backward compatibility we
-    // don't add a group separator for C locale.
+    // Group digits. For backward compatibility, we skip this for the C locale.
     if (locale != QLocale::c() && !locale.numberOptions().testFlag(QLocale::OmitGroupSeparator))
         flags |= QLocaleData::GroupDigits;
 
     const QLocaleData *dd = locale.d->m_data;
     int base = params.integerBase ? params.integerBase : 10;
-    if (negative && base == 10) {
-        result = dd->longLongToString(-static_cast<qlonglong>(number), -1,
-                                      base, -1, flags);
-    } else if (negative) {
-        // Workaround for backward compatibility for writing negative
-        // numbers in octal and hex:
-        // QTextStream(result) << Qt::showbase << Qt::hex << -1 << oct << -1
-        // should output: -0x1 -0b1
-        result = dd->unsLongLongToString(number, -1, base, -1, flags);
+    QString result = dd->unsLongLongToString(number, -1, base, -1, flags);
+    if (negative) {
         result.prepend(locale.negativeSign());
-    } else {
-        result = dd->unsLongLongToString(number, -1, base, -1, flags);
-        // workaround for backward compatibility - in octal form with
-        // ShowBase flag set zero should be written as '00'
-        if (number == 0 && base == 8 && params.numberFlags & QTextStream::ShowBase
-            && result == "0"_L1) {
-            result.prepend(u'0');
-        }
+    } else if (number == 0 && base == 8 && params.numberFlags & QTextStream::ShowBase
+               && result == "0"_L1) {
+        // Workaround for backward compatibility - in octal form with ShowBase
+        // flag set, zero should get its 0 prefix before its 0 value, but
+        // QLocalePrivate only adds the prefix if the number doesn't start with
+        // a zero.
+        result.prepend(u'0');
     }
     putString(result, true);
 }
@@ -2294,7 +2284,7 @@ QTextStream &QTextStream::operator<<(signed short i)
 {
     Q_D(QTextStream);
     CHECK_VALID_STREAM(*this);
-    d->putNumber((qulonglong)qAbs(qlonglong(i)), i < 0);
+    d->putNumber(QtPrivate::qUnsignedAbs(i), i < 0);
     return *this;
 }
 
@@ -2320,7 +2310,7 @@ QTextStream &QTextStream::operator<<(signed int i)
 {
     Q_D(QTextStream);
     CHECK_VALID_STREAM(*this);
-    d->putNumber((qulonglong)qAbs(qlonglong(i)), i < 0);
+    d->putNumber(QtPrivate::qUnsignedAbs(i), i < 0);
     return *this;
 }
 
@@ -2346,7 +2336,7 @@ QTextStream &QTextStream::operator<<(signed long i)
 {
     Q_D(QTextStream);
     CHECK_VALID_STREAM(*this);
-    d->putNumber((qulonglong)qAbs(qlonglong(i)), i < 0);
+    d->putNumber(QtPrivate::qUnsignedAbs(i), i < 0);
     return *this;
 }
 
@@ -2372,7 +2362,7 @@ QTextStream &QTextStream::operator<<(qlonglong i)
 {
     Q_D(QTextStream);
     CHECK_VALID_STREAM(*this);
-    d->putNumber((qulonglong)qAbs(i), i < 0);
+    d->putNumber(QtPrivate::qUnsignedAbs(i), i < 0);
     return *this;
 }
 
@@ -2833,7 +2823,7 @@ QTextStream &center(QTextStream &stream)
     \snippet code/src_corelib_io_qtextstream.cpp 9
 
     Note: On Windows, all '\\n' characters are written as '\\r\\n' if
-    QTextStream's device or string is opened using the QIODevice::Text flag.
+    QTextStream's device or string is opened using the \l QIODeviceBase::Text flag.
 
     \since 5.14
 
