@@ -32,8 +32,8 @@
     The template parameter \c T must be a C++ integer type:
     \list
        \li 8-bit: bool, char, signed char, unsigned char, qint8, quint8, char8_t (C++20)
-       \li 16-bit: short, unsigned short, qint16, quint16, char16_t (C++11)
-       \li 32-bit: int, unsigned int, qint32, quint32, char32_t (C++11)
+       \li 16-bit: short, unsigned short, qint16, quint16, char16_t
+       \li 32-bit: int, unsigned int, qint32, quint32, char32_t
        \li 64-bit: long long, unsigned long long, qint64, quint64
        \li platform-specific size: long, unsigned long
        \li pointer size: qintptr, quintptr, qptrdiff
@@ -397,14 +397,21 @@
 
     Atomic test-and-set.
 
+    \note If you use this function in a loop, consider using the overload with the
+    additional \c{T &currentValue} argument instead, which avoids the extra loadRelaxed() on
+    failure.
+
     If the current value of this QAtomicInteger is the \a expectedValue,
     the test-and-set functions assign the \a newValue to this
     QAtomicInteger and return true. If the values are \e not the same,
     this function does nothing and returns \c false.
 
+//![memory-order-relaxed]
     This function uses \e relaxed \l {QAtomicInteger#Memory
     ordering}{memory ordering} semantics, leaving the compiler and
     processor to freely reorder memory accesses.
+//![memory-order-relaxed]
+
 */
 
 /*!
@@ -412,15 +419,21 @@
 
     Atomic test-and-set.
 
+    \note If you use this function in a loop, consider using the overload with the
+    additional \c{T &currentValue} argument instead, which avoids the extra loadAcquire() on
+    failure.
+
     If the current value of this QAtomicInteger is the \a expectedValue,
     the test-and-set functions assign the \a newValue to this
     QAtomicInteger and return true. If the values are \e not the same,
     this function does nothing and returns \c false.
 
+//![memory-order-acquire]
     This function uses \e acquire \l {QAtomicInteger#Memory
     ordering}{memory ordering} semantics, which ensures that memory
     access following the atomic operation (in program order) may not
     be re-ordered before the atomic operation.
+//![memory-order-acquire]
 */
 
 /*!
@@ -428,15 +441,21 @@
 
     Atomic test-and-set.
 
+    \note If you use this function in a loop, consider using the overload with the
+    additional \c{T &currentValue} argument instead, which avoids the extra loadRelaxed() on
+    failure.
+
     If the current value of this QAtomicInteger is the \a expectedValue,
     the test-and-set functions assign the \a newValue to this
     QAtomicInteger and return true. If the values are \e not the same,
     this function does nothing and returns \c false.
 
+//![memory-order-release]
     This function uses \e release \l {QAtomicInteger#Memory
     ordering}{memory ordering} semantics, which ensures that memory
     access before the atomic operation (in program order) may not be
     re-ordered after the atomic operation.
+//![memory-order-release]
 */
 
 /*!
@@ -444,15 +463,78 @@
 
     Atomic test-and-set.
 
+    \note If you use this function in a loop, consider using the overload with the
+    additional \c{T &currentValue} argument instead, which avoids the extra loadAcquire() on
+    failure.
+
     If the current value of this QAtomicInteger is the \a expectedValue,
     the test-and-set functions assign the \a newValue to this
     QAtomicInteger and return true. If the values are \e not the same,
     this function does nothing and returns \c false.
 
+//![memory-order-ordered]
     This function uses \e ordered \l {QAtomicInteger#Memory
     ordering}{memory ordering} semantics, which ensures that memory
     access before and after the atomic operation (in program order)
     may not be re-ordered.
+//![memory-order-ordered]
+
+*/
+
+/*!
+    \fn template <typename T> bool QAtomicInteger<T>::testAndSetRelaxed(T expectedValue, T newValue, T &currentValue)
+    \since 5.3
+
+    Atomic test-and-set.
+
+    If the current value of this QAtomicInteger is the \a expectedValue, the
+    test-and-set functions assign the \a newValue to this QAtomicInteger and
+    return \c true. If the values are \e not the same, the functions load the
+    current value of this QAtomicInteger into \a currentValue and return \c false.
+
+    \include qatomic.cpp memory-order-relaxed
+*/
+
+/*!
+    \fn template <typename T> bool QAtomicInteger<T>::testAndSetAcquire(T expectedValue, T newValue, T &currentValue)
+    \since 5.3
+
+    Atomic test-and-set.
+
+    If the current value of this QAtomicInteger is the \a expectedValue, the
+    test-and-set functions assign the \a newValue to this QAtomicInteger and
+    return \c true. If the values are \e not the same, the functions load the
+    current value of this QAtomicInteger into \a currentValue and return \c false.
+
+    \include qatomic.cpp memory-order-acquire
+*/
+
+/*!
+    \fn template <typename T> bool QAtomicInteger<T>::testAndSetRelease(T expectedValue, T newValue, T &currentValue)
+    \since 5.3
+
+    Atomic test-and-set.
+
+    If the current value of this QAtomicInteger is the \a expectedValue, the
+    test-and-set functions assign the \a newValue to this QAtomicInteger and
+    return \c true. If the values are \e not the same, the functions loads the
+    current value of this QAtomicInteger into \a currentValue and return \c false.
+
+    \include qatomic.cpp memory-order-release
+*/
+
+/*!
+    \fn template <typename T> bool QAtomicInteger<T>::testAndSetOrdered(T expectedValue, T newValue, T &currentValue)
+    \since 5.3
+
+    Atomic test-and-set.
+
+    If the current value of this QAtomicInteger is the \a expectedValue, the
+    test-and-set functions assign the \a newValue to this QAtomicInteger and
+    return \c true. If the values are \e not the same, it loads the current
+    value of this QAtomicInteger into \a currentValue and return \c false.
+
+    \include qatomic.cpp memory-order-ordered
 */
 
 /*!
@@ -978,7 +1060,7 @@
 
     This macro is defined when only certain generations of the
     processor support atomic reference counting. Use the
-    QAtomicInteger<T>::isReferenceCountingNative() function to check what
+    QAtomicInteger::isReferenceCountingNative() function to check what
     your processor supports.
 
     \e{nn} is the size of the integer, in bits (8, 16, 32 or 64).
@@ -1021,7 +1103,7 @@
 
     This macro is defined when only certain generations of the
     processor support atomic test-and-set on integers. Use the
-    QAtomicInteger<T>::isTestAndSetNative() function to check what your
+    QAtomicInteger::isTestAndSetNative() function to check what your
     processor supports.
 
     \e{nn} is the size of the integer, in bits (8, 16, 32 or 64).
@@ -1064,7 +1146,7 @@
 
     This macro is defined when only certain generations of the
     processor support atomic fetch-and-store on integers. Use the
-    QAtomicInteger<T>::isFetchAndStoreNative() function to check what your
+    QAtomicInteger::isFetchAndStoreNative() function to check what your
     processor supports.
 
     \e{nn} is the size of the integer, in bits (8, 16, 32 or 64).
@@ -1107,7 +1189,7 @@
 
     This macro is defined when only certain generations of the
     processor support atomic fetch-and-add on integers. Use the
-    QAtomicInteger<T>::isFetchAndAddNative() function to check what your
+    QAtomicInteger::isFetchAndAddNative() function to check what your
     processor supports.
 
     \e{nn} is the size of the integer, in bits (8, 16, 32 or 64).
@@ -1277,7 +1359,7 @@
 
     \endlist
 
-    \sa QAtomicInteger
+    \sa QAtomicInteger, qYieldCpu()
 */
 
 /*!
