@@ -1,5 +1,6 @@
 // Copyright (C) 2016 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:critical reason:data-parser
 #ifndef QT_NO_ICON
 #include <private/qiconloader_p.h>
 
@@ -9,7 +10,7 @@
 #include <QtGui/QIconEnginePlugin>
 #include <QtGui/QPixmapCache>
 #include <qpa/qplatformtheme.h>
-#include <QtGui/QIconEngine>
+#include <QtGui/qfontdatabase.h>
 #include <QtGui/QPalette>
 #include <QtCore/qmath.h>
 #include <QtCore/QList>
@@ -21,10 +22,12 @@
 #include <QtGui/QPainter>
 
 #include <private/qhexstring_p.h>
+#include <private/qfactoryloader_p.h>
+#include <private/qfonticonengine_p.h>
 
 QT_BEGIN_NAMESPACE
 
-Q_LOGGING_CATEGORY(lcIconLoader, "qt.gui.icon.loader")
+Q_STATIC_LOGGING_CATEGORY(lcIconLoader, "qt.gui.icon.loader")
 
 using namespace Qt::StringLiterals;
 
@@ -48,9 +51,8 @@ QIconLoader::QIconLoader() :
 
 static inline QString systemThemeName()
 {
-    const auto override = qgetenv("QT_QPA_SYSTEM_ICON_THEME");
-    if (!override.isEmpty())
-        return QString::fromLocal8Bit(override);
+    if (QString override = qEnvironmentVariable("QT_QPA_SYSTEM_ICON_THEME"); !override.isEmpty())
+        return override;
     if (const QPlatformTheme *theme = QGuiApplicationPrivate::platformTheme()) {
         const QVariant themeHint = theme->themeHint(QPlatformTheme::SystemIconThemeName);
         if (themeHint.isValid())
@@ -137,6 +139,10 @@ void QIconLoader::invalidateKey()
     // recreating the actual engine the next time the icon is used.
     // We don't need to clear the QIcon cache itself.
     m_themeKey++;
+
+    // invalidating the factory results in us looking once for
+    // a plugin that provides icon for the new themeName()
+    m_factory = std::nullopt;
 }
 
 QString QIconLoader::themeName() const
@@ -649,13 +655,47 @@ QIconEngine *QIconLoader::iconEngine(const QString &iconName) const
 {
     qCDebug(lcIconLoader) << "Resolving icon engine for icon" << iconName;
 
-    auto *platformTheme = QGuiApplicationPrivate::platformTheme();
     std::unique_ptr<QIconEngine> iconEngine;
-    if (!hasUserTheme() && platformTheme)
-        iconEngine.reset(platformTheme->createIconEngine(iconName));
-    if (!iconEngine || iconEngine->isNull()) {
-        iconEngine.reset(new QIconLoaderEngine(iconName));
+
+    if (!m_factory) {
+        qCDebug(lcIconLoader) << "Finding a plugin for theme" << themeName();
+        // try to find a plugin that supports the current theme
+        const int factoryIndex = qt_iconEngineFactoryLoader()->indexOf(themeName());
+        if (factoryIndex >= 0)
+            m_factory = qobject_cast<QIconEnginePlugin *>(qt_iconEngineFactoryLoader()->instance(factoryIndex));
     }
+    if (m_factory && *m_factory)
+        iconEngine.reset(m_factory.value()->create(iconName));
+
+    if (hasUserTheme()) {
+        if (!iconEngine || iconEngine->isNull()) {
+            if (QFontDatabase::families().contains(themeName())) {
+                QFont maybeIconFont(themeName());
+                maybeIconFont.setStyleStrategy(QFont::NoFontMerging);
+                qCDebug(lcIconLoader) << "Trying font icon engine.";
+                iconEngine.reset(new QFontIconEngine(iconName, maybeIconFont));
+            }
+        }
+        if (!iconEngine || iconEngine->isNull()) {
+            qCDebug(lcIconLoader) << "Trying loader engine for theme.";
+            iconEngine.reset(new QIconLoaderEngine(iconName));
+        }
+    }
+
+    if (!iconEngine || iconEngine->isNull()) {
+        qCDebug(lcIconLoader) << "Icon is not available from theme or fallback theme.";
+        if (auto *platformTheme = QGuiApplicationPrivate::platformTheme()) {
+            qCDebug(lcIconLoader) << "Trying platform engine.";
+            std::unique_ptr<QIconEngine> themeEngine(platformTheme->createIconEngine(iconName));
+            if (themeEngine && !themeEngine->isNull()) {
+                iconEngine = std::move(themeEngine);
+                qCDebug(lcIconLoader) << "Icon provided by platform engine.";
+            }
+        }
+    }
+    // We need to maintain the invariant that the QIcon has a valid engine
+    if (!iconEngine)
+        iconEngine.reset(new QIconLoaderEngine(iconName));
 
     qCDebug(lcIconLoader) << "Resulting engine" << iconEngine.get();
     return iconEngine.release();
@@ -758,31 +798,28 @@ bool QIconLoaderEngine::hasIcon() const
 void QIconLoaderEngine::paint(QPainter *painter, const QRect &rect,
                              QIcon::Mode mode, QIcon::State state)
 {
-    QSize pixmapSize = rect.size() * painter->device()->devicePixelRatio();
-    painter->drawPixmap(rect, pixmap(pixmapSize, mode, state));
+    const auto dpr = painter->device()->devicePixelRatio();
+    painter->drawPixmap(rect, scaledPixmap(rect.size(), mode, state, dpr));
 }
 
 /*
  * This algorithm is defined by the freedesktop spec:
  * http://standards.freedesktop.org/icon-theme-spec/icon-theme-spec-latest.html
  */
-static bool directoryMatchesSize(const QIconDirInfo &dir, int iconsize, int iconscale)
+static bool directoryMatchesSizeAndScale(const QIconDirInfo &dir, int iconsize, int iconscale)
 {
     if (dir.scale != iconscale)
         return false;
 
-    if (dir.type == QIconDirInfo::Fixed) {
+    switch (dir.type) {
+    case QIconDirInfo::Fixed:
         return dir.size == iconsize;
-
-    } else if (dir.type == QIconDirInfo::Scalable) {
-        return iconsize <= dir.maxSize &&
-                iconsize >= dir.minSize;
-
-    } else if (dir.type == QIconDirInfo::Threshold) {
-        return iconsize >= dir.size - dir.threshold &&
-                iconsize <= dir.size + dir.threshold;
-    } else if (dir.type == QIconDirInfo::Fallback) {
-        return true;
+    case QIconDirInfo::Scalable:
+        return iconsize <= dir.maxSize && iconsize >= dir.minSize;
+    case QIconDirInfo::Threshold:
+        return iconsize >= dir.size - dir.threshold && iconsize <= dir.size + dir.threshold;
+    case QIconDirInfo::Fallback:
+        return false;   // just because the scale matches it doesn't mean there is a better sized icon somewhere
     }
 
     Q_ASSERT(1); // Not a valid value
@@ -790,31 +827,33 @@ static bool directoryMatchesSize(const QIconDirInfo &dir, int iconsize, int icon
 }
 
 /*
- * This algorithm is defined by the freedesktop spec:
+ * This algorithm is a modification of the algorithm defined by the freedesktop spec:
  * http://standards.freedesktop.org/icon-theme-spec/icon-theme-spec-latest.html
  */
-static int directorySizeDistance(const QIconDirInfo &dir, int iconsize, int iconscale)
+static std::optional<int> directorySizeDelta(const QIconDirInfo &dir, int iconsize, int iconscale)
 {
-    const int scaledIconSize = iconsize * iconscale;
-    if (dir.type == QIconDirInfo::Fixed) {
-        return qAbs(dir.size * dir.scale - scaledIconSize);
+    const auto scaledIconSize = iconsize * iconscale;
 
-    } else if (dir.type == QIconDirInfo::Scalable) {
-        if (scaledIconSize < dir.minSize * dir.scale)
-            return dir.minSize * dir.scale - scaledIconSize;
-        else if (scaledIconSize > dir.maxSize * dir.scale)
-            return scaledIconSize - dir.maxSize * dir.scale;
-        else
-            return 0;
-
-    } else if (dir.type == QIconDirInfo::Threshold) {
+    switch (dir.type) {
+    case QIconDirInfo::Fixed:
+        return dir.size * dir.scale - scaledIconSize;
+    case QIconDirInfo::Scalable: {
+        const auto minScaled = dir.minSize * dir.scale;
+        if (scaledIconSize < minScaled)
+            return minScaled - scaledIconSize;
+        const auto maxScaled = dir.maxSize * dir.scale;
+        if (scaledIconSize > maxScaled)
+            return scaledIconSize - maxScaled;
+        return {};
+    }
+    case QIconDirInfo::Threshold:
         if (scaledIconSize < (dir.size - dir.threshold) * dir.scale)
             return dir.minSize * dir.scale - scaledIconSize;
-        else if (scaledIconSize > (dir.size + dir.threshold) * dir.scale)
+        if (scaledIconSize > (dir.size + dir.threshold) * dir.scale)
             return scaledIconSize - dir.maxSize * dir.scale;
-        else return 0;
-    } else if (dir.type == QIconDirInfo::Fallback) {
-        return 0;
+        return {};
+    case QIconDirInfo::Fallback:
+        return {};
     }
 
     Q_ASSERT(1); // Not a valid value
@@ -823,29 +862,43 @@ static int directorySizeDistance(const QIconDirInfo &dir, int iconsize, int icon
 
 QIconLoaderEngineEntry *QIconLoaderEngine::entryForSize(const QThemeIconInfo &info, const QSize &size, int scale)
 {
+    if (info.entries.empty())
+        return nullptr;
+    if (info.entries.size() == 1)
+        return info.entries.at(0).get();
+
     int iconsize = qMin(size.width(), size.height());
 
     // Note that m_info.entries are sorted so that png-files
     // come first
 
-    // Search for exact matches first
-    for (const auto &entry : info.entries) {
-        if (directoryMatchesSize(entry->dir, iconsize, scale)) {
-            return entry.get();
-        }
-    }
-
-    // Find the minimum distance icon
-    int minimalSize = INT_MAX;
+    int minimalDelta = INT_MIN;
     QIconLoaderEngineEntry *closestMatch = nullptr;
     for (const auto &entry : info.entries) {
-        int distance = directorySizeDistance(entry->dir, iconsize, scale);
-        if (distance < minimalSize) {
-            minimalSize  = distance;
-            closestMatch = entry.get();
+        // exact match in scale and dpr
+        if (directoryMatchesSizeAndScale(entry->dir, iconsize, scale))
+            return entry.get();
+
+        // Find the minimum distance icon
+        const auto delta = directorySizeDelta(entry->dir, iconsize, scale);
+        if (delta.has_value()) {
+            const auto deltaValue = delta.value();
+            // always prefer downscaled icons over upscaled icons
+            if (deltaValue > minimalDelta && minimalDelta <= 0) {
+                minimalDelta = deltaValue;
+                closestMatch = entry.get();
+            } else if (deltaValue > 0 && deltaValue < qAbs(minimalDelta)) {
+                minimalDelta = deltaValue;
+                closestMatch = entry.get();
+            } else if (deltaValue == 0) {
+                // exact match but different dpr:
+                // --> size * scale == entry.size * entry.scale
+                minimalDelta = deltaValue;
+                closestMatch = entry.get();
+            }
         }
     }
-    return closestMatch;
+    return closestMatch ? closestMatch : info.entries.at(0).get();
 }
 
 /*
@@ -875,7 +928,7 @@ QSize QIconLoaderEngine::actualSize(const QSize &size, QIcon::Mode mode,
     return QSize(0, 0);
 }
 
-QPixmap PixmapEntry::pixmap(const QSize &size, QIcon::Mode mode, QIcon::State state)
+QPixmap PixmapEntry::pixmap(const QSize &size, QIcon::Mode mode, QIcon::State state, qreal scale)
 {
     Q_UNUSED(state);
 
@@ -884,18 +937,17 @@ QPixmap PixmapEntry::pixmap(const QSize &size, QIcon::Mode mode, QIcon::State st
     if (basePixmap.isNull())
         basePixmap.load(filename);
 
-    QSize actualSize = basePixmap.size();
     // If the size of the best match we have (basePixmap) is larger than the
     // requested size, we downscale it to match.
-    if (!actualSize.isNull() && (actualSize.width() > size.width() || actualSize.height() > size.height()))
-        actualSize.scale(size, Qt::KeepAspectRatio);
-
+    const auto actualSize = QPixmapIconEngine::adjustSize(size * scale, basePixmap.size());
+    const auto calculatedDpr = QIconPrivate::pixmapDevicePixelRatio(scale, size, actualSize);
     QString key = "$qt_theme_"_L1
-                  % HexString<qint64>(basePixmap.cacheKey())
-                  % HexString<int>(mode)
-                  % HexString<qint64>(QGuiApplication::palette().cacheKey())
-                  % HexString<int>(actualSize.width())
-                  % HexString<int>(actualSize.height());
+                  % HexString<quint64>(basePixmap.cacheKey())
+                  % HexString<quint8>(mode)
+                  % HexString<quint64>(QGuiApplication::palette().cacheKey())
+                  % HexString<uint>(actualSize.width())
+                  % HexString<uint>(actualSize.height())
+                  % HexString<quint16>(qRound(calculatedDpr * 1000));
 
     QPixmap cachedPixmap;
     if (QPixmapCache::find(key, &cachedPixmap)) {
@@ -907,32 +959,24 @@ QPixmap PixmapEntry::pixmap(const QSize &size, QIcon::Mode mode, QIcon::State st
             cachedPixmap = basePixmap;
         if (QGuiApplication *guiApp = qobject_cast<QGuiApplication *>(qApp))
             cachedPixmap = static_cast<QGuiApplicationPrivate*>(QObjectPrivate::get(guiApp))->applyQIconStyleHelper(mode, cachedPixmap);
+        cachedPixmap.setDevicePixelRatio(calculatedDpr);
         QPixmapCache::insert(key, cachedPixmap);
     }
     return cachedPixmap;
 }
 
-QPixmap ScalableEntry::pixmap(const QSize &size, QIcon::Mode mode, QIcon::State state)
+QPixmap ScalableEntry::pixmap(const QSize &size, QIcon::Mode mode, QIcon::State state, qreal scale)
 {
     if (svgIcon.isNull())
         svgIcon = QIcon(filename);
 
-    // Bypass QIcon API, as that will scale by device pixel ratio of the
-    // highest DPR screen since we're not passing on any QWindow.
-    if (QIconEngine *engine = svgIcon.data_ptr() ? svgIcon.data_ptr()->engine : nullptr)
-        return engine->pixmap(size, mode, state);
-
-    return QPixmap();
+    return svgIcon.pixmap(size, scale, mode, state);
 }
 
 QPixmap QIconLoaderEngine::pixmap(const QSize &size, QIcon::Mode mode,
                                  QIcon::State state)
 {
-    QIconLoaderEngineEntry *entry = entryForSize(m_info, size);
-    if (entry)
-        return entry->pixmap(size, mode, state);
-
-    return QPixmap();
+    return scaledPixmap(size, mode, state, 1.0);
 }
 
 QString QIconLoaderEngine::key() const
@@ -953,8 +997,8 @@ bool QIconLoaderEngine::isNull()
 QPixmap QIconLoaderEngine::scaledPixmap(const QSize &size, QIcon::Mode mode, QIcon::State state, qreal scale)
 {
     const int integerScale = qCeil(scale);
-    QIconLoaderEngineEntry *entry = entryForSize(m_info, size / integerScale, integerScale);
-    return entry ? entry->pixmap(size, mode, state) : QPixmap();
+    QIconLoaderEngineEntry *entry = entryForSize(m_info, size, integerScale);
+    return entry ? entry->pixmap(size, mode, state, scale) : QPixmap();
 }
 
 QList<QSize> QIconLoaderEngine::availableSizes(QIcon::Mode mode, QIcon::State state)
