@@ -1,41 +1,7 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtNetwork module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+
+#include <QtNetwork/private/qtnetworkglobal_p.h>
 
 #include "qnetworkreply.h"
 #include "qnetworkreply_p.h"
@@ -43,6 +9,8 @@
 
 QT_BEGIN_NAMESPACE
 
+QT_IMPL_METATYPE_EXTERN_TAGGED(QNetworkReply::NetworkError, QNetworkReply__NetworkError)
+
 const int QNetworkReplyPrivate::progressSignalInterval = 100;
 
 QNetworkReplyPrivate::QNetworkReplyPrivate()
@@ -61,7 +29,7 @@ QNetworkReplyPrivate::QNetworkReplyPrivate()
     \class QNetworkReply
     \since 4.4
     \brief The QNetworkReply class contains the data and headers for a request
-    sent with QNetworkAccessManager
+    sent with QNetworkAccessManager.
 
     \reentrant
     \ingroup network
@@ -74,7 +42,7 @@ QNetworkReplyPrivate::QNetworkReplyPrivate()
     itself.
 
     QNetworkReply is a sequential-access QIODevice, which means that
-    once data is read from the object, it no longer kept by the
+    once data is read from the object, it is no longer kept by the
     device. It is therefore the application's responsibility to keep
     this data if it needs to. Whenever more data is received from the
     network and processed, the readyRead() signal is emitted.
@@ -91,7 +59,7 @@ QNetworkReplyPrivate::QNetworkReplyPrivate()
     content.
 
     \note Do not delete the object in the slot connected to the
-    error() or finished() signal. Use deleteLater().
+    errorOccurred() or finished() signal. Use deleteLater().
 
     \sa QNetworkRequest, QNetworkAccessManager
 */
@@ -219,6 +187,7 @@ QNetworkReplyPrivate::QNetworkReplyPrivate()
     the server response was detected
 
     \sa error()
+    \sa errorOccurred()
 */
 
 /*!
@@ -295,13 +264,13 @@ QNetworkReplyPrivate::QNetworkReplyPrivate()
     \fn void QNetworkReply::redirected(const QUrl &url)
     \since 5.6
 
-    This signal is emitted if the QNetworkRequest::FollowRedirectsAttribute was
+    This signal is emitted if the QNetworkRequest::ManualRedirectPolicy was not
     set in the request and the server responded with a 3xx status (specifically
     301, 302, 303, 305, 307 or 308 status code) with a valid url in the location
     header, indicating a HTTP redirect. The \a url parameter contains the new
     redirect url as returned by the server in the location header.
 
-    \sa QNetworkRequest::FollowRedirectsAttribute
+    \sa QNetworkRequest::RedirectPolicy
 */
 
 /*!
@@ -318,6 +287,27 @@ QNetworkReplyPrivate::QNetworkReplyPrivate()
     QNetworkRequest::RedirectPolicyAttribute
 */
 
+/*!
+    \fn void QNetworkReply::socketStartedConnecting()
+    \since 6.3
+
+    This signal is emitted 0 or more times, when the socket
+    is connecting, before sending the request. Useful for
+    custom progress or timeout handling.
+
+    \sa metaDataChanged(), requestSent()
+*/
+
+/*!
+    \fn void QNetworkReply::requestSent()
+    \since 6.3
+
+    This signal is emitted 1 or more times when the request was
+    sent. Useful for custom progress or timeout handling.
+
+    \sa metaDataChanged(), socketStartedConnecting()
+*/
+
 /*!
     \fn void QNetworkReply::metaDataChanged()
 
@@ -341,7 +331,7 @@ QNetworkReplyPrivate::QNetworkReplyPrivate()
     processing. After this signal is emitted, there will be no more
     updates to the reply's data or metadata.
 
-    Unless close() or abort() have been called, the reply will be still be opened
+    Unless close() or abort() have been called, the reply will still be opened
     for reading, so the data can be retrieved by calls to read() or
     readAll(). In particular, if no calls to read() were made as a
     result of readyRead(), a call to readAll() will retrieve the full
@@ -361,7 +351,8 @@ QNetworkReplyPrivate::QNetworkReplyPrivate()
 */
 
 /*!
-    \fn void QNetworkReply::error(QNetworkReply::NetworkError code)
+    \fn void QNetworkReply::errorOccurred(QNetworkReply::NetworkError code)
+    \since 5.15
 
     This signal is emitted when the reply detects an error in
     processing. The finished() signal will probably follow, indicating
@@ -442,7 +433,7 @@ QNetworkReplyPrivate::QNetworkReplyPrivate()
     QNetworkAccessManager functions to do that.
 */
 QNetworkReply::QNetworkReply(QObject *parent)
-    : QIODevice(*new QNetworkReplyPrivate, parent)
+    : QNetworkReply(*new QNetworkReplyPrivate, parent)
 {
 }
 
@@ -592,10 +583,10 @@ bool QNetworkReply::isRunning() const
 
 /*!
     Returns the URL of the content downloaded or uploaded. Note that
-    the URL may be different from that of the original request. If the
-    QNetworkRequest::FollowRedirectsAttribute was set in the request, then this
+    the URL may be different from that of the original request.
+    If redirections were enabled in the request, then this
     function returns the current url that the network API is accessing, i.e the
-    url emitted in the QNetworkReply::redirected signal.
+    url of the resource the request got redirected to.
 
     \sa request(), setUrl(), QNetworkRequest::url(), redirected()
 */
@@ -621,11 +612,12 @@ QVariant QNetworkReply::header(QNetworkRequest::KnownHeaders header) const
     the remote server
 
     \sa rawHeader()
+    \note In Qt versions prior to 6.7, this function took QByteArray only.
 */
-bool QNetworkReply::hasRawHeader(const QByteArray &headerName) const
+bool QNetworkReply::hasRawHeader(QAnyStringView headerName) const
 {
     Q_D(const QNetworkReply);
-    return d->findRawHeader(headerName) != d->rawHeaders.constEnd();
+    return d->headers().contains(headerName);
 }
 
 /*!
@@ -636,20 +628,17 @@ bool QNetworkReply::hasRawHeader(const QByteArray &headerName) const
     header field.
 
     \sa setRawHeader(), hasRawHeader(), header()
+    \note In Qt versions prior to 6.7, this function took QByteArray only.
 */
-QByteArray QNetworkReply::rawHeader(const QByteArray &headerName) const
+QByteArray QNetworkReply::rawHeader(QAnyStringView headerName) const
 {
     Q_D(const QNetworkReply);
-    QNetworkHeadersPrivate::RawHeadersList::ConstIterator it =
-        d->findRawHeader(headerName);
-    if (it != d->rawHeaders.constEnd())
-        return it->second;
-    return QByteArray();
+    return d->rawHeader(headerName);
 }
 
 /*! \typedef QNetworkReply::RawHeaderPair
 
-  RawHeaderPair is a QPair<QByteArray, QByteArray> where the first
+  RawHeaderPair is a std::pair<QByteArray, QByteArray> where the first
   QByteArray is the header name and the second is the header.
  */
 
@@ -659,7 +648,20 @@ QByteArray QNetworkReply::rawHeader(const QByteArray &headerName) const
 const QList<QNetworkReply::RawHeaderPair>& QNetworkReply::rawHeaderPairs() const
 {
     Q_D(const QNetworkReply);
-    return d->rawHeaders;
+    return d->allRawHeaders();
+}
+
+/*!
+    \since 6.8
+
+    Returns headers that were sent by the remote server.
+
+    \sa setHeaders(), QNetworkRequest::setAttribute(), QNetworkRequest::Attribute
+*/
+QHttpHeaders QNetworkReply::headers() const
+{
+    Q_D(const QNetworkReply);
+    return d->headers();
 }
 
 /*!
@@ -687,7 +689,7 @@ QVariant QNetworkReply::attribute(QNetworkRequest::Attribute code) const
     return d_func()->attributes.value(code);
 }
 
-#ifndef QT_NO_SSL
+#if QT_CONFIG(ssl)
 /*!
     Returns the SSL configuration and state associated with this
     reply, if SSL was used. It will contain the remote server's
@@ -742,7 +744,6 @@ void QNetworkReply::ignoreSslErrors(const QList<QSslError> &errors)
 {
     ignoreSslErrorsImplementation(errors);
 }
-#endif
 
 /*!
   \fn void QNetworkReply::sslConfigurationImplementation(QSslConfiguration &configuration) const
@@ -786,6 +787,8 @@ void QNetworkReply::ignoreSslErrorsImplementation(const QList<QSslError> &)
 {
 }
 
+#endif // QT_CONFIG(ssl)
+
 /*!
     If this function is called, SSL errors related to network
     connection will be ignored, including certificate validation
@@ -854,7 +857,7 @@ void QNetworkReply::setRequest(const QNetworkRequest &request)
     Sets the error condition to be \a errorCode. The human-readable
     message is set with \a errorString.
 
-    Calling setError() does not emit the error(QNetworkReply::NetworkError)
+    Calling setError() does not emit the errorOccurred(QNetworkReply::NetworkError)
     signal.
 
     \sa error(), errorString()
@@ -895,6 +898,45 @@ void QNetworkReply::setUrl(const QUrl &url)
     d->url = url;
 }
 
+/*!
+    \since 6.8
+
+    Sets \a newHeaders as headers in this network reply, overriding
+    any previously set headers.
+
+    If some headers correspond to the known headers, they will be
+    parsed and the corresponding parsed form will also be set.
+
+    \sa headers(), QNetworkRequest::KnownHeaders
+*/
+void QNetworkReply::setHeaders(const QHttpHeaders &newHeaders)
+{
+    Q_D(QNetworkReply);
+    d->setHeaders(newHeaders);
+}
+
+/*!
+    \overload
+    \since 6.8
+*/
+void QNetworkReply::setHeaders(QHttpHeaders &&newHeaders)
+{
+    Q_D(QNetworkReply);
+    d->setHeaders(std::move(newHeaders));
+}
+
+/*!
+    \since 6.8
+
+    Sets the header \a name to be of value \a value. If \a
+    name was previously set, it is overridden.
+*/
+void QNetworkReply::setWellKnownHeader(QHttpHeaders::WellKnownHeader name, QByteArrayView value)
+{
+    Q_D(QNetworkReply);
+    d->setHeader(name, value);
+}
+
 /*!
     Sets the known header \a header to be of value \a value. The
     corresponding raw form of the header will be set as well.
@@ -941,3 +983,5 @@ void QNetworkReply::setAttribute(QNetworkRequest::Attribute code, const QVariant
 }
 
 QT_END_NAMESPACE
+
+#include "moc_qnetworkreply.cpp"
