@@ -1,41 +1,6 @@
-/****************************************************************************
-**
-** Copyright (C) 2022 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the plugins of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2022 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:critical reason:data-parser
 
 /*!
     \class QtIcoHandler
@@ -535,7 +500,7 @@ QImage ICOReader::iconAt(int index)
                     if (!image.isNull()) {
                         readBMP(image);
                         if (!image.isNull()) {
-                            if (icoAttrib.depth == 32) {
+                            if (icoAttrib.nbits == 32) {
                                 img = std::move(image).convertToFormat(QImage::Format_ARGB32_Premultiplied);
                             } else {
                                 QImage mask(image.width(), image.height(), QImage::Format_Mono);
@@ -601,14 +566,14 @@ bool ICOReader::write(QIODevice *device, const QList<QImage> &images)
 {
     bool retValue = false;
 
-    if (images.count()) {
+    if (images.size()) {
 
         qint64 origOffset = device->pos();
 
         ICONDIR id;
         id.idReserved = 0;
         id.idType = 1;
-        id.idCount = images.count();
+        id.idCount = images.size();
 
         ICONDIRENTRY * entries = new ICONDIRENTRY[id.idCount];
         BMP_INFOHDR * bmpHeaders = new BMP_INFOHDR[id.idCount];
@@ -771,6 +736,8 @@ bool QtIcoHandler::supportsOption(ImageOption option) const
  */
 bool QtIcoHandler::canRead() const
 {
+    if (knownCanRead)
+        return true;
     bool bCanRead = false;
     QIODevice *device = QImageIOHandler::device();
     if (device) {
@@ -780,6 +747,7 @@ bool QtIcoHandler::canRead() const
     } else {
         qCWarning(lcIco, "QtIcoHandler::canRead() called with no device");
     }
+    knownCanRead = bCanRead;
     return bCanRead;
 }
 
