@@ -1,13 +1,15 @@
 // Copyright (C) 2016 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
+#undef QT_NO_FOREACH // this file contains unported legacy Q_FOREACH uses
+
 #include "qiosintegration.h"
 #include "qioseventdispatcher.h"
 #include "qiosglobal.h"
 #include "qioswindow.h"
 #include "qiosscreen.h"
 #include "qiosplatformaccessibility.h"
-#ifndef Q_OS_TVOS
+#if QT_CONFIG(clipboard)
 #include "qiosclipboard.h"
 #endif
 #include "qiosinputcontext.h"
@@ -15,6 +17,10 @@
 #include "qiosservices.h"
 #include "qiosoptionalplugininterface.h"
 
+#if defined(Q_OS_VISIONOS)
+#include "qiosswiftintegration.h"
+#endif
+
 #include <QtGui/qpointingdevice.h>
 #include <QtGui/private/qguiapplication_p.h>
 #include <QtGui/private/qrhibackingstore_p.h>
@@ -49,11 +55,10 @@ QIOSIntegration *QIOSIntegration::instance()
 
 QIOSIntegration::QIOSIntegration()
     : m_fontDatabase(new QCoreTextFontDatabaseEngineFactory<QCoreTextFontEngine>)
-#if !defined(Q_OS_TVOS) && !defined(QT_NO_CLIPBOARD)
+#if QT_CONFIG(clipboard)
     , m_clipboard(new QIOSClipboard)
 #endif
     , m_inputContext(0)
-    , m_platformServices(new QIOSServices)
     , m_accessibility(0)
     , m_optionalPlugins(new QFactoryLoader(QIosOptionalPluginInterface_iid, "/platforms/darwin"_L1))
 {
@@ -70,6 +75,10 @@ QIOSIntegration::QIOSIntegration()
 
 void QIOSIntegration::initialize()
 {
+#if defined(Q_OS_VISIONOS)
+    // Qt requires a screen, so let's give it a dummy one
+    QWindowSystemInterface::handleScreenAdded(new QIOSScreen);
+#else
     UIScreen *mainScreen = [UIScreen mainScreen];
     NSMutableArray<UIScreen *> *screens = [[[UIScreen screens] mutableCopy] autorelease];
     if (![screens containsObject:mainScreen]) {
@@ -79,16 +88,18 @@ void QIOSIntegration::initialize()
 
     for (UIScreen *screen in screens)
         QWindowSystemInterface::handleScreenAdded(new QIOSScreen(screen));
+#endif
 
     // Depends on a primary screen being present
     m_inputContext = new QIOSInputContext;
 
-    m_touchDevice = new QPointingDevice;
-    m_touchDevice->setType(QInputDevice::DeviceType::TouchScreen);
     QPointingDevice::Capabilities touchCapabilities = QPointingDevice::Capability::Position | QPointingDevice::Capability::NormalizedPosition;
+#if !defined(Q_OS_VISIONOS)
     if (mainScreen.traitCollection.forceTouchCapability == UIForceTouchCapabilityAvailable)
         touchCapabilities |= QPointingDevice::Capability::Pressure;
-    m_touchDevice->setCapabilities(touchCapabilities);
+#endif
+    m_touchDevice = new QPointingDevice("touchscreen", 0, QInputDevice::DeviceType::TouchScreen,
+                                QPointingDevice::PointerType::Finger, touchCapabilities, 10, 0);
     QWindowSystemInterface::registerInputDevice(m_touchDevice);
 #if QT_CONFIG(tabletevent)
     QWindowSystemInterfacePrivate::TabletEvent::setPlatformSynthesizesMouse(false);
@@ -105,7 +116,7 @@ QIOSIntegration::~QIOSIntegration()
     delete m_fontDatabase;
     m_fontDatabase = 0;
 
-#if !defined(Q_OS_TVOS) && !defined(QT_NO_CLIPBOARD)
+#if QT_CONFIG(clipboard)
     delete m_clipboard;
     m_clipboard = 0;
 #endif
@@ -206,14 +217,10 @@ QPlatformFontDatabase * QIOSIntegration::fontDatabase() const
     return m_fontDatabase;
 }
 
-#ifndef QT_NO_CLIPBOARD
+#if QT_CONFIG(clipboard)
 QPlatformClipboard *QIOSIntegration::clipboard() const
 {
-#ifndef Q_OS_TVOS
     return m_clipboard;
-#else
-    return QPlatformIntegration::clipboard();
-#endif
 }
 #endif
 
@@ -224,6 +231,9 @@ QPlatformInputContext *QIOSIntegration::inputContext() const
 
 QPlatformServices *QIOSIntegration::services() const
 {
+    if (!m_platformServices)
+        m_platformServices = new QIOSServices;
+
     return m_platformServices;
 }
 
@@ -291,6 +301,52 @@ void QIOSIntegration::setApplicationBadge(qint64 number)
 
 // ---------------------------------------------------------
 
+#if defined(Q_OS_VISIONOS)
+void QIOSIntegration::openImmersiveSpace()
+{
+    [ImmersiveSpaceManager openImmersiveSpace];
+}
+
+void QIOSIntegration::dismissImmersiveSpace()
+{
+    [ImmersiveSpaceManager dismissImmersiveSpace];
+}
+
+void QIOSIntegration::setImmersiveSpaceCompositorLayer(CompositorLayer *layer)
+{
+    m_immersiveSpaceCompositorLayer = layer;
+}
+
+void QIOSIntegration::configureCompositorLayer(cp_layer_renderer_capabilities_t capabilities,
+                                               cp_layer_renderer_configuration_t configuration)
+{
+    if (m_immersiveSpaceCompositorLayer)
+        m_immersiveSpaceCompositorLayer->configure(capabilities, configuration);
+}
+
+void QIOSIntegration::renderCompositorLayer(cp_layer_renderer_t renderer)
+{
+    if (m_immersiveSpaceCompositorLayer)
+        m_immersiveSpaceCompositorLayer->render(renderer);
+}
+
+void QIOSIntegration::handleSpatialEvents(const char *jsonString)
+{
+    if (m_immersiveSpaceCompositorLayer) {
+        QJsonParseError error;
+        QJsonDocument doc = QJsonDocument::fromJson(QByteArray(jsonString), &error);
+        if (error.error != QJsonParseError::NoError) {
+            qWarning() << "Error parsing JSON: " << error.errorString();
+            return;
+        }
+        m_immersiveSpaceCompositorLayer->handleSpatialEvents(doc.object());
+    }
+}
+
+#endif
+
+// ---------------------------------------------------------
+
 void *QIOSIntegration::nativeResourceForWindow(const QByteArray &resource, QWindow *window)
 {
     if (!window || !window->handle())
