@@ -33,10 +33,12 @@
 #include <stdlib.h>
 #include <string.h>
 #include <limits.h>
-#include <vector>
 
-#include <vector>
+#include <QtCore/q20algorithm.h>
+#include <atomic>
+#include <cstdio>
 #include <memory>
+#include <vector>
 
 QT_BEGIN_NAMESPACE
 
@@ -67,10 +69,72 @@ static void saveCoverageTool(const char * appname, bool testfailed, bool install
 #endif
 }
 
-static QElapsedTimer elapsedFunctionTime;
-static QElapsedTimer elapsedTotalTime;
+Q_CONSTINIT static QBasicMutex elapsedTimersMutex; // due to the WatchDog thread
+Q_CONSTINIT static QElapsedTimer elapsedFunctionTime;
+Q_CONSTINIT static QElapsedTimer elapsedTotalTime;
 
-#define FOREACH_TEST_LOGGER for (const auto &logger : std::as_const(*QTest::loggers()))
+namespace {
+class LoggerRegistry
+{
+    using LoggersContainer = std::vector<std::shared_ptr<QAbstractTestLogger>>;
+    using SharedLoggersContainer = std::shared_ptr<LoggersContainer>;
+
+public:
+    void addLogger(std::unique_ptr<QAbstractTestLogger> logger)
+    {
+        // read/update/clone
+        const SharedLoggersContainer currentLoggers = load();
+        SharedLoggersContainer newLoggers = currentLoggers
+                ? std::make_shared<LoggersContainer>(*currentLoggers)
+                : std::make_shared<LoggersContainer>();
+        newLoggers->emplace_back(std::move(logger));
+        store(std::move(newLoggers));
+    }
+
+    void clear() { store(SharedLoggersContainer{}); }
+
+    auto allLoggers() const
+    {
+        struct LoggersRange
+        {
+            const SharedLoggersContainer loggers;
+
+            auto begin() const
+            {
+                return loggers ? loggers->cbegin() : LoggersContainer::const_iterator{};
+            }
+            auto end() const
+            {
+                return loggers ? loggers->cend() : LoggersContainer::const_iterator{};
+            }
+            bool isEmpty() const { return loggers ? loggers->empty() : true; }
+        };
+
+        return LoggersRange{ load() };
+    }
+
+private:
+#ifdef __cpp_lib_atomic_shared_ptr
+    SharedLoggersContainer load() const { return loggers.load(std::memory_order_relaxed); }
+    void store(SharedLoggersContainer newLoggers)
+    {
+        loggers.store(std::move(newLoggers), std::memory_order_relaxed);
+    }
+    std::atomic<SharedLoggersContainer> loggers;
+#else
+    SharedLoggersContainer load() const
+    {
+        return std::atomic_load_explicit(&loggers, std::memory_order_relaxed);
+    }
+    void store(SharedLoggersContainer newLoggers)
+    {
+        std::atomic_store_explicit(&loggers, std::move(newLoggers), std::memory_order_relaxed);
+    }
+    SharedLoggersContainer loggers;
+#endif
+};
+
+} // namespace
 
 namespace QTest {
 
@@ -143,7 +207,7 @@ namespace QTest {
 
     static std::vector<QVariant> failOnWarningList;
 
-    Q_GLOBAL_STATIC(std::vector<std::unique_ptr<QAbstractTestLogger>>, loggers)
+    Q_GLOBAL_STATIC(LoggerRegistry, loggers)
 
     static int verbosity = 0;
     static int maxWarnings = 2002;
@@ -164,10 +228,8 @@ namespace QTest {
                 // remove the item from the list
                 if (last)
                     last->next = list->next;
-                else if (list->next)
-                    ignoreResultList = list->next;
                 else
-                    ignoreResultList = nullptr;
+                    ignoreResultList = list->next;
 
                 delete list;
                 return true;
@@ -181,7 +243,7 @@ namespace QTest {
 
     static bool handleFailOnWarning(const QMessageLogContext &context, const QString &message)
     {
-        // failOnWarnings can be called multiple times per test function, so let
+        // failOnWarning can be called multiple times per test function, so let
         // each call cause a failure if required.
         for (const auto &pattern : failOnWarningList) {
             if (pattern.metaType() == QMetaType::fromType<QString>()) {
@@ -197,8 +259,8 @@ namespace QTest {
 
             const size_t maxMsgLen = 1024;
             char msg[maxMsgLen] = {'\0'};
-            qsnprintf(msg, maxMsgLen, "Received a warning that resulted in a failure:\n%s",
-                      qPrintable(message));
+            std::snprintf(msg, maxMsgLen, "Received a warning that resulted in a failure:\n%s",
+                          qPrintable(message));
             QTestResult::addFailure(msg, context.file, context.line);
             return true;
         }
@@ -209,10 +271,14 @@ namespace QTest {
     {
         static QBasicAtomicInt counter = Q_BASIC_ATOMIC_INITIALIZER(QTest::maxWarnings);
 
-        if (!QTestLog::hasLoggers()) {
-            // if this goes wrong, something is seriously broken.
-            qInstallMessageHandler(oldMessageHandler);
-            QTEST_ASSERT(QTestLog::hasLoggers());
+        auto loggerCapture = loggers->allLoggers();
+
+        if (loggerCapture.isEmpty()) {
+            // the message handler may be called from a worker thread, after the main thread stopped
+            // logging. Forwarding to original message handler to avoid swallowing the message
+            Q_ASSERT(oldMessageHandler);
+            oldMessageHandler(type, context, message);
+            return;
         }
 
         if (handleIgnoredMessage(type, message)) {
@@ -228,19 +294,20 @@ namespace QTest {
                 return;
 
             if (!counter.deref()) {
-                FOREACH_TEST_LOGGER {
+                for (auto &logger : loggerCapture)
                     logger->addMessage(QAbstractTestLogger::Warn,
-                        QStringLiteral("Maximum amount of warnings exceeded. Use -maxwarnings to override."));
-                }
+                                       QStringLiteral("Maximum amount of warnings exceeded. Use "
+                                                      "-maxwarnings to override."));
+
                 return;
             }
         }
 
-        FOREACH_TEST_LOGGER
+        for (auto &logger : loggerCapture)
             logger->addMessage(type, context, message);
 
         if (type == QtFatalMsg) {
-             /* Right now, we're inside the custom message handler and we're
+            /* Right now, we're inside the custom message handler and we're
              * being qt_message_output in qglobal.cpp. After we return from
              * this function, it will proceed with calling exit() and abort()
              * and hence crash. Therefore, we call these logging functions such
@@ -254,13 +321,16 @@ namespace QTest {
 
 void QTestLog::enterTestFunction(const char* function)
 {
-    elapsedFunctionTime.restart();
+    {
+        QMutexLocker locker(&elapsedTimersMutex);
+        elapsedFunctionTime.start();
+    }
     if (printAvailableTags)
         return;
 
     QTEST_ASSERT(function);
 
-    FOREACH_TEST_LOGGER
+    for (auto &logger : QTest::loggers->allLoggers())
         logger->enterTestFunction(function);
 }
 
@@ -268,7 +338,7 @@ void QTestLog::enterTestData(QTestData *data)
 {
     QTEST_ASSERT(data);
 
-    FOREACH_TEST_LOGGER
+    for (auto &logger : QTest::loggers->allLoggers())
         logger->enterTestData(data);
 }
 
@@ -289,7 +359,7 @@ void QTestLog::leaveTestFunction()
     if (printAvailableTags)
         return;
 
-    FOREACH_TEST_LOGGER
+    for (auto &logger : QTest::loggers->allLoggers())
         logger->leaveTestFunction();
 }
 
@@ -307,7 +377,7 @@ void QTestLog::printUnhandledIgnoreMessages()
                     list->pattern.toRegularExpression().pattern());
 #endif
         }
-        FOREACH_TEST_LOGGER
+        for (auto &logger : QTest::loggers->allLoggers())
             logger->addMessage(QAbstractTestLogger::Info, message);
 
         list = list->next;
@@ -320,7 +390,6 @@ void QTestLog::clearIgnoreMessages()
     QTest::IgnoreResultList::clearList(QTest::ignoreResultList);
 }
 
-
 void QTestLog::clearFailOnWarnings()
 {
     QTest::failOnWarningList.clear();
@@ -328,6 +397,8 @@ void QTestLog::clearFailOnWarnings()
 
 void QTestLog::clearCurrentTestState()
 {
+    clearIgnoreMessages();
+    clearFailOnWarnings();
     QTest::currentTestState = QTest::Unresolved;
 }
 
@@ -342,7 +413,7 @@ void QTestLog::addPass(const char *msg)
     ++QTest::passes;
     QTest::currentTestState = QTest::Passed;
 
-    FOREACH_TEST_LOGGER
+    for (auto &logger : QTest::loggers->allLoggers())
         logger->addIncident(QAbstractTestLogger::Pass, msg);
 }
 
@@ -362,7 +433,7 @@ void QTestLog::addFail(const char *msg, const char *file, int line)
     // subsequent failures; they may carry useful information.
 
     QTest::currentTestState = QTest::Failed;
-    FOREACH_TEST_LOGGER
+    for (auto &logger : QTest::loggers->allLoggers())
         logger->addIncident(QAbstractTestLogger::Fail, msg, file, line);
 }
 
@@ -372,7 +443,7 @@ void QTestLog::addXFail(const char *msg, const char *file, int line)
 
     // Will be counted in addPass() if we get there.
 
-    FOREACH_TEST_LOGGER
+    for (auto &logger : QTest::loggers->allLoggers())
         logger->addIncident(QAbstractTestLogger::XFail, msg, file, line);
 }
 
@@ -390,7 +461,7 @@ void QTestLog::addXPass(const char *msg, const char *file, int line)
     }
 
     QTest::currentTestState = QTest::Failed;
-    FOREACH_TEST_LOGGER
+    for (auto &logger : QTest::loggers->allLoggers())
         logger->addIncident(QAbstractTestLogger::XPass, msg, file, line);
 }
 
@@ -402,7 +473,7 @@ void QTestLog::addBPass(const char *msg)
     ++QTest::blacklists; // Not passes ?
     QTest::currentTestState = QTest::Suppressed;
 
-    FOREACH_TEST_LOGGER
+    for (auto &logger : QTest::loggers->allLoggers())
         logger->addIncident(QAbstractTestLogger::BlacklistedPass, msg);
 }
 
@@ -420,7 +491,7 @@ void QTestLog::addBFail(const char *msg, const char *file, int line)
     }
 
     QTest::currentTestState = QTest::Suppressed;
-    FOREACH_TEST_LOGGER
+    for (auto &logger : QTest::loggers->allLoggers())
         logger->addIncident(QAbstractTestLogger::BlacklistedFail, msg, file, line);
 }
 
@@ -438,7 +509,7 @@ void QTestLog::addBXPass(const char *msg, const char *file, int line)
     }
 
     QTest::currentTestState = QTest::Suppressed;
-    FOREACH_TEST_LOGGER
+    for (auto &logger : QTest::loggers->allLoggers())
         logger->addIncident(QAbstractTestLogger::BlacklistedXPass, msg, file, line);
 }
 
@@ -448,7 +519,7 @@ void QTestLog::addBXFail(const char *msg, const char *file, int line)
 
     // Will be counted in addBPass() if we get there.
 
-    FOREACH_TEST_LOGGER
+    for (auto &logger : QTest::loggers->allLoggers())
         logger->addIncident(QAbstractTestLogger::BlacklistedXFail, msg, file, line);
 }
 
@@ -469,21 +540,24 @@ void QTestLog::addSkip(const char *msg, const char *file, int line)
     // It is up to particular loggers to decide whether to report such
     // subsequent skips; they may carry useful information.
 
-    FOREACH_TEST_LOGGER
+    for (auto &logger : QTest::loggers->allLoggers())
         logger->addIncident(QAbstractTestLogger::Skip, msg, file, line);
 }
 
-void QTestLog::addBenchmarkResult(const QBenchmarkResult &result)
+void QTestLog::addBenchmarkResults(const QList<QBenchmarkResult> &results)
 {
-    FOREACH_TEST_LOGGER
-        logger->addBenchmarkResult(result);
+    for (auto &logger : QTest::loggers->allLoggers())
+        logger->addBenchmarkResults(results);
 }
 
 void QTestLog::startLogging()
 {
-    elapsedTotalTime.start();
-    elapsedFunctionTime.start();
-    FOREACH_TEST_LOGGER
+    {
+        QMutexLocker locker(&elapsedTimersMutex);
+        elapsedTotalTime.start();
+        elapsedFunctionTime.start();
+    }
+    for (auto &logger : QTest::loggers->allLoggers())
         logger->startLogging();
     QTest::oldMessageHandler = qInstallMessageHandler(QTest::messageHandler);
 }
@@ -491,10 +565,10 @@ void QTestLog::startLogging()
 void QTestLog::stopLogging()
 {
     qInstallMessageHandler(QTest::oldMessageHandler);
-    FOREACH_TEST_LOGGER {
+    for (auto &logger : QTest::loggers->allLoggers())
         logger->stopLogging();
-    }
-    QTest::loggers()->clear();
+
+    QTest::loggers->clear();
     saveCoverageTool(QTestResult::currentAppName(), failCount() != 0, QTestLog::installedTestCoverage());
 }
 
@@ -539,7 +613,7 @@ void QTestLog::addLogger(LogMode mode, const char *filename)
     }
 
     QTEST_ASSERT(logger);
-    addLogger(logger);
+    addLogger(std::unique_ptr<QAbstractTestLogger>{ logger });
 }
 
 /*!
@@ -547,35 +621,45 @@ void QTestLog::addLogger(LogMode mode, const char *filename)
 
     Adds a new logger to the set of loggers that will be used
     to report incidents and messages during testing.
-
-    The function takes ownership of the logger.
 */
-void QTestLog::addLogger(QAbstractTestLogger *logger)
+void QTestLog::addLogger(std::unique_ptr<QAbstractTestLogger> logger)
 {
     QTEST_ASSERT(logger);
-    QTest::loggers()->emplace_back(logger);
+    QTest::loggers()->addLogger(std::move(logger));
 }
 
 bool QTestLog::hasLoggers()
 {
-    return !QTest::loggers()->empty();
+    return !QTest::loggers()->allLoggers().isEmpty();
+}
+
+/*!
+    \internal
+
+    Returns true if all loggers support repeated test runs
+*/
+bool QTestLog::isRepeatSupported()
+{
+    for (auto &logger : QTest::loggers->allLoggers())
+        if (!logger->isRepeatSupported())
+            return false;
+
+    return true;
 }
 
 bool QTestLog::loggerUsingStdout()
 {
-    FOREACH_TEST_LOGGER {
-        if (logger->isLoggingToStdout())
-            return true;
-    }
-
-    return false;
+    auto loggersCapture = QTest::loggers->allLoggers();
+    return q20::ranges::any_of(loggersCapture.begin(), loggersCapture.end(), [](auto &logger) {
+        return logger->isLoggingToStdout();
+    });
 }
 
 void QTestLog::warn(const char *msg, const char *file, int line)
 {
     QTEST_ASSERT(msg);
 
-    FOREACH_TEST_LOGGER
+    for (auto &logger : QTest::loggers->allLoggers())
         logger->addMessage(QAbstractTestLogger::Warn, QString::fromUtf8(msg), file, line);
 }
 
@@ -583,7 +667,7 @@ void QTestLog::info(const char *msg, const char *file, int line)
 {
     QTEST_ASSERT(msg);
 
-    FOREACH_TEST_LOGGER
+    for (auto &logger : QTest::loggers->allLoggers())
         logger->addMessage(QAbstractTestLogger::Info, QString::fromUtf8(msg), file, line);
 }
 
@@ -615,6 +699,11 @@ void QTestLog::ignoreMessage(QtMsgType type, const QRegularExpression &expressio
 }
 #endif // QT_CONFIG(regularexpression)
 
+void QTestLog::failOnWarning()
+{
+    QTest::failOnWarningList.push_back({});
+}
+
 void QTestLog::failOnWarning(const char *msg)
 {
     QTest::failOnWarningList.push_back(QString::fromUtf8(msg));
@@ -685,11 +774,13 @@ bool QTestLog::installedTestCoverage()
 
 qint64 QTestLog::nsecsTotalTime()
 {
+    QMutexLocker locker(&elapsedTimersMutex);
     return elapsedTotalTime.nsecsElapsed();
 }
 
 qint64 QTestLog::nsecsFunctionTime()
 {
+    QMutexLocker locker(&elapsedTimersMutex);
     return elapsedFunctionTime.nsecsElapsed();
 }
 
