@@ -923,8 +923,13 @@ void QCompleterPrivate::showPopup(const QRect& rect)
 
     popup->setGeometry(pos.x(), pos.y(), w, h);
 
-    if (!popup->isVisible())
+    if (!popup->isVisible()) {
+        // Make sure popup has a transient parent set, Wayland needs it. QTBUG-130474
+        popup->winId(); // force creation of windowHandle
+        popup->windowHandle()->setTransientParent(widget->window()->windowHandle());
+
         popup->show();
+    }
 }
 
 #if QT_CONFIG(filesystemmodel)
@@ -1370,11 +1375,13 @@ bool QCompleter::eventFilter(QObject *o, QEvent *e)
             return false;
         }
 
-        // Send the event to the widget. If the widget accepted the event, do nothing
-        // If the widget did not accept the event, provide a default implementation
-        d->eatFocusOut = false;
-        (static_cast<QObject *>(d->widget))->event(ke);
-        d->eatFocusOut = true;
+        if (d->widget) {
+            // Send the event to the widget. If the widget accepted the event, do nothing
+            // If the widget did not accept the event, provide a default implementation
+            d->eatFocusOut = false;
+            (static_cast<QObject *>(d->widget))->event(ke);
+            d->eatFocusOut = true;
+        }
         if (!d->widget || e->isAccepted() || !d->popup->isVisible()) {
             // widget lost focus, hide the popup
             if (d->widget && (!d->widget->hasFocus()
@@ -1426,8 +1433,9 @@ bool QCompleter::eventFilter(QObject *o, QEvent *e)
 
 #ifdef QT_KEYPAD_NAVIGATION
     case QEvent::KeyRelease: {
-        QKeyEvent *ke = static_cast<QKeyEvent *>(e);
-        if (QApplicationPrivate::keypadNavigationEnabled() && ke->key() == Qt::Key_Back) {
+        if (d->widget &&
+            QApplicationPrivate::keypadNavigationEnabled() && ke->key() == Qt::Key_Back) {
+            QKeyEvent *ke = static_cast<QKeyEvent *>(e);
             // Send the event to the 'widget'. This is what we did for KeyPress, so we need
             // to do the same for KeyRelease, in case the widget's KeyPress event set
             // up something (such as a timer) that is relying on also receiving the
@@ -1444,7 +1452,8 @@ bool QCompleter::eventFilter(QObject *o, QEvent *e)
 
     case QEvent::MouseButtonPress: {
 #ifdef QT_KEYPAD_NAVIGATION
-        if (QApplicationPrivate::keypadNavigationEnabled()) {
+        if (d->widget
+            && QApplicationPrivate::keypadNavigationEnabled()) {
             // if we've clicked in the widget (or its descendant), let it handle the click
             QWidget *source = qobject_cast<QWidget *>(o);
             if (source) {
@@ -1473,7 +1482,8 @@ bool QCompleter::eventFilter(QObject *o, QEvent *e)
         return true;
     case QEvent::InputMethod:
     case QEvent::ShortcutOverride:
-        QCoreApplication::sendEvent(d->widget, e);
+        if (d->widget)
+            QCoreApplication::sendEvent(d->widget, e);
         break;
 
     default:
