@@ -1,8 +1,8 @@
 // Copyright (C) 2016 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
-#ifndef QDYNAMICMAINWINDOWLAYOUT_P_H
-#define QDYNAMICMAINWINDOWLAYOUT_P_H
+#ifndef QMAINWINDOWLAYOUT_P_H
+#define QMAINWINDOWLAYOUT_P_H
 
 //
 //  W A R N I N G
@@ -29,15 +29,25 @@
 #include "QtCore/qset.h"
 #include "private/qlayoutengine_p.h"
 #include "private/qwidgetanimator_p.h"
-
 #if QT_CONFIG(dockwidget)
+#include "private/qdockwidget_p.h"
+
 #include "qdockarealayout_p.h"
 #include "qdockwidget.h"
+#else
+struct QDockWidgetPrivate {
+    enum class DragScope {
+        Group
+    };
+};
 #endif
 #if QT_CONFIG(toolbar)
 #include "qtoolbararealayout_p.h"
+#include "qtoolbar.h"
 #endif
+
 #include <QtCore/qloggingcategory.h>
+#include <QtCore/qpointer.h>
 
 QT_REQUIRE_CONFIG(mainwindow);
 
@@ -83,6 +93,9 @@ public:
     bool endSeparatorMove(const QPoint &pos);
     bool windowEvent(QEvent *e);
 
+private:
+    QList<int> findSeparator(const QPoint &pos) const;
+
 #endif // QT_CONFIG(dockwidget)
 
 };
@@ -133,7 +146,7 @@ void QMainWindowLayoutSeparatorHelper<Layout>::adjustCursor(const QPoint &pos)
                 w->unsetCursor();
         }
     } else if (movingSeparator.isEmpty()) { // Don't change cursor when moving separator
-        QList<int> pathToSeparator = layout()->dockAreaLayoutInfo()->findSeparator(pos);
+        QList<int> pathToSeparator = findSeparator(pos);
 
         if (pathToSeparator != hoverSeparator) {
             if (!hoverSeparator.isEmpty())
@@ -270,10 +283,35 @@ bool QMainWindowLayoutSeparatorHelper<Layout>::windowEvent(QEvent *event)
     return false;
 }
 
+template <typename Layout>
+QList<int> QMainWindowLayoutSeparatorHelper<Layout>::findSeparator(const QPoint &pos) const
+{
+    Layout *layout = const_cast<Layout*>(this->layout());
+#if QT_CONFIG(toolbar)
+    QToolBarAreaLayout *toolBarAreaLayout = layout->toolBarAreaLayout();
+    if (toolBarAreaLayout && !toolBarAreaLayout->isEmpty()) {
+        // We might have a toolbar that is currently expanded, covering
+        // parts of the dock area, in which case we don't want the dock
+        // area layout to treat mouse events for the expanded toolbar as
+        // hitting a separator.
+        const QWidget *widget = layout->window();
+        QWidget *childWidget = widget->childAt(pos);
+        while (childWidget && childWidget != widget) {
+            if (auto *toolBar = qobject_cast<QToolBar*>(childWidget)) {
+                if (!toolBarAreaLayout->indexOf(toolBar).isEmpty())
+                    return {};
+            }
+            childWidget = childWidget->parentWidget();
+        }
+    }
+#endif
+    return layout->dockAreaLayoutInfo()->findSeparator(pos);
+}
+
 template <typename Layout>
 bool QMainWindowLayoutSeparatorHelper<Layout>::startSeparatorMove(const QPoint &pos)
 {
-    movingSeparator = layout()->dockAreaLayoutInfo()->findSeparator(pos);
+    movingSeparator = findSeparator(pos);
 
     if (movingSeparator.isEmpty())
         return false;
@@ -306,8 +344,10 @@ class Q_AUTOTEST_EXPORT QDockWidgetGroupWindow : public QWidget
 {
     Q_OBJECT
 public:
-    explicit QDockWidgetGroupWindow(QWidget* parent = nullptr, Qt::WindowFlags f = { })
-        : QWidget(parent, f) {}
+    explicit QDockWidgetGroupWindow(QWidget *parent = nullptr, Qt::WindowFlags f = {})
+        : QWidget(parent, f)
+    {
+    }
     QDockAreaLayoutInfo *layoutInfo() const;
 #if QT_CONFIG(tabbar)
     const QDockAreaLayoutInfo *tabLayoutInfo() const;
@@ -322,6 +362,10 @@ public:
     void updateCurrentGapRect();
     void restore();
     void apply();
+    void childEvent(QChildEvent *event) override;
+    void reparentToMainWindow(QDockWidget *dockWidget);
+    void destroyIfSingleItemLeft();
+    QList<QDockWidget *> dockWidgets() const { return findChildren<QDockWidget *>(); }
 
     QRect currentGapRect;
     QList<int> currentGapPos;
@@ -331,6 +375,7 @@ signals:
 
 protected:
     bool event(QEvent *) override;
+    bool eventFilter(QObject *obj, QEvent *event) override;
     void paintEvent(QPaintEvent*) override;
 
 private:
@@ -434,6 +479,7 @@ public:
     bool restoreState(QDataStream &stream, const QMainWindowLayoutState &oldState);
 };
 
+class QMainWindowTabBar;
 class Q_AUTOTEST_EXPORT QMainWindowLayout
     : public QLayout,
       public QMainWindowLayoutSeparatorHelper<QMainWindowLayout>
@@ -476,6 +522,7 @@ public:
     void removeToolBar(QToolBar *toolbar);
     void toggleToolBarsVisible();
     void moveToolBar(QToolBar *toolbar, int pos);
+    QToolBarAreaLayout *toolBarAreaLayout() { return &layoutState.toolBarAreaLayout; }
 #endif
 
     // dock widgets
@@ -490,7 +537,7 @@ public:
     void splitDockWidget(QDockWidget *after,
                          QDockWidget *dockwidget,
                          Qt::Orientation orientation);
-    Qt::DockWidgetArea dockWidgetArea(QWidget* widget) const;
+    Qt::DockWidgetArea dockWidgetArea(const QWidget* widget) const;
     bool restoreDockWidget(QDockWidget *dockwidget);
 #if QT_CONFIG(tabbar)
     void tabifyDockWidget(QDockWidget *first, QDockWidget *second);
@@ -561,17 +608,29 @@ public:
 #if QT_CONFIG(dockwidget)
     QPointer<QDockWidgetGroupWindow> currentHoveredFloat; // set when dragging over a floating dock widget
     void setCurrentHoveredFloat(QDockWidgetGroupWindow *w);
+#if QT_CONFIG(tabbar)
+    bool isDockWidgetTabbed(const QDockWidget *dockWidget) const;
+    QList<QDockWidget *> tabifiedDockWidgets(const QDockWidget *dockWidget) const;
+    QMainWindowTabBar *findTabBar(const QDockWidget *dockWidget) const;
+#endif
 #endif
     bool isInApplyState = false;
 
     void hover(QLayoutItem *hoverTarget, const QPoint &mousePos);
     bool plug(QLayoutItem *widgetItem);
-    QLayoutItem *unplug(QWidget *widget, bool group = false);
+    QLayoutItem *unplug(QWidget *widget, QDockWidgetPrivate::DragScope scope);
     void revert(QLayoutItem *widgetItem);
     void applyState(QMainWindowLayoutState &newState, bool animate = true);
+    void applyRestoredState();
     void restore(bool keepSavedState = false);
     void animationFinished(QWidget *widget);
 
+#if QT_CONFIG(draganddrop)
+    static bool needsPlatformDrag();
+    Qt::DropAction performPlatformWidgetDrag(QLayoutItem *widgetItem, const QPoint &pressPosition);
+    QLayoutItem *draggingWidget = nullptr;
+#endif
+
 protected:
     void timerEvent(QTimerEvent *e) override;
 
@@ -598,4 +657,4 @@ QDebug operator<<(QDebug debug, const QMainWindowLayout *layout);
 
 QT_END_NAMESPACE
 
-#endif // QDYNAMICMAINWINDOWLAYOUT_P_H
+#endif // QMAINWINDOWLAYOUT_P_H
