@@ -1,6 +1,5 @@
 // Copyright (C) 2021 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
-
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #include <QTest>
 
@@ -24,6 +23,8 @@
 #include <QStandardPaths>
 #endif
 
+using namespace Qt::StringLiterals;
+
 Q_DECLARE_METATYPE(QDirIterator::IteratorFlags)
 Q_DECLARE_METATYPE(QDir::Filters)
 
@@ -45,16 +46,10 @@ private: // convenience functions
         return false;
     }
 
-    enum Cleanup { DoDelete, DontDelete };
-    bool createFile(const QString &fileName, Cleanup cleanup = DoDelete)
+    bool createFile(const QString &fileName)
     {
         QFile file(fileName);
-        if (file.open(QIODevice::WriteOnly)) {
-            if (cleanup == DoDelete)
-                createdFiles << fileName;
-            return true;
-        }
-        return false;
+        return file.open(QIODevice::WriteOnly);
     }
 
     bool createLink(const QString &destination, const QString &linkName)
@@ -68,7 +63,6 @@ private: // convenience functions
 
 private slots:
     void initTestCase();
-    void cleanupTestCase();
     void iterateRelativeDirectory_data();
     void iterateRelativeDirectory();
     void iterateResource_data();
@@ -76,6 +70,8 @@ private slots:
     void stopLinkLoop();
 #ifdef QT_BUILD_INTERNAL
     void engineWithNoIterator();
+    void testQFsFileEngineIterator_data() { iterateRelativeDirectory_data(); }
+    void testQFsFileEngineIterator();
 #endif
     void absoluteFilePathsFromRelativeIteratorPath();
     void recurseWithFilters() const;
@@ -89,16 +85,18 @@ private slots:
 #ifndef Q_OS_WIN
     void hiddenDirs_hiddenFiles();
 #endif
-#ifdef BUILTIN_TESTDATA
+
+    void hasNextFalseNoCrash();
+
 private:
     QSharedPointer<QTemporaryDir> m_dataDir;
-#endif
 };
 
 void tst_QDirIterator::initTestCase()
 {
+    QString testdata_dir;
 #ifdef Q_OS_ANDROID
-    QString testdata_dir = QStandardPaths::writableLocation(QStandardPaths::CacheLocation);
+    testdata_dir = QStandardPaths::writableLocation(QStandardPaths::CacheLocation);
     QString resourceSourcePath = QStringLiteral(":/testdata");
     QDirIterator it(resourceSourcePath, QDirIterator::Subdirectories);
     while (it.hasNext()) {
@@ -121,29 +119,22 @@ void tst_QDirIterator::initTestCase()
 #elif defined(BUILTIN_TESTDATA)
     m_dataDir = QEXTRACTTESTDATA("/testdata");
     QVERIFY2(!m_dataDir.isNull(), qPrintable("Could not extract test data"));
-    QString testdata_dir = m_dataDir->path();
+    testdata_dir = m_dataDir->path();
 #else
-
-    // chdir into testdata directory, then find testdata by relative paths.
-    QString testdata_dir = QFileInfo(QFINDTESTDATA("entrylist")).absolutePath();
+    m_dataDir.reset(new QTemporaryDir);
+    testdata_dir = m_dataDir->path();
 #endif
 
+    QVERIFY(!testdata_dir.isEmpty());
+    // Must call QDir::setCurrent() here because all the tests that use relative
+    // paths depend on that.
     QVERIFY2(QDir::setCurrent(testdata_dir), qPrintable("Could not chdir to " + testdata_dir));
 
-    QFile::remove("entrylist/entrylist1.lnk");
-    QFile::remove("entrylist/entrylist2.lnk");
-    QFile::remove("entrylist/entrylist3.lnk");
-    QFile::remove("entrylist/entrylist4.lnk");
-    QFile::remove("entrylist/directory/entrylist1.lnk");
-    QFile::remove("entrylist/directory/entrylist2.lnk");
-    QFile::remove("entrylist/directory/entrylist3.lnk");
-    QFile::remove("entrylist/directory/entrylist4.lnk");
-
     createDirectory("entrylist");
     createDirectory("entrylist/directory");
-    createFile("entrylist/file", DontDelete);
+    createFile("entrylist/file");
     createFile("entrylist/writable");
-    createFile("entrylist/directory/dummy", DontDelete);
+    createFile("entrylist/directory/dummy");
 
     createDirectory("recursiveDirs");
     createDirectory("recursiveDirs/dir1");
@@ -191,15 +182,6 @@ void tst_QDirIterator::initTestCase()
 #endif
 }
 
-void tst_QDirIterator::cleanupTestCase()
-{
-    Q_FOREACH(QString fileName, createdFiles)
-        QFile::remove(fileName);
-
-    Q_FOREACH(QString dirName, createdDirectories)
-        currentDir.rmdir(dirName);
-}
-
 void tst_QDirIterator::iterateRelativeDirectory_data()
 {
     QTest::addColumn<QString>("dirName"); // relative from current path or abs
@@ -324,7 +306,7 @@ void tst_QDirIterator::iterateRelativeDirectory()
     QFETCH(QDirIterator::IteratorFlags, flags);
     QFETCH(QDir::Filters, filters);
     QFETCH(QStringList, nameFilters);
-    QFETCH(QStringList, entries);
+    QFETCH(const QStringList, entries);
 
     QDirIterator it(dirName, nameFilters, filters, flags);
     QStringList list;
@@ -352,13 +334,13 @@ void tst_QDirIterator::iterateRelativeDirectory()
     list.sort();
 
     QStringList sortedEntries;
-    foreach(QString item, entries)
+    for (const QString &item : entries)
         sortedEntries.append(QFileInfo(item).canonicalFilePath());
     sortedEntries.sort();
 
     if (sortedEntries != list) {
-        qDebug() << "EXPECTED:" << sortedEntries;
         qDebug() << "ACTUAL:  " << list;
+        qDebug() << "EXPECTED:" << sortedEntries;
     }
 
     QCOMPARE(list, sortedEntries);
@@ -375,13 +357,14 @@ void tst_QDirIterator::iterateResource_data()
     QTest::newRow("invalid") << QString::fromLatin1(":/testdata/burpaburpa") << QDirIterator::IteratorFlags{}
                              << QDir::Filters(QDir::NoFilter) << QStringList(QLatin1String("*"))
                              << QStringList();
-    QTest::newRow(":/testdata") << QString::fromLatin1(":/testdata/") << QDirIterator::IteratorFlags{}
+    QTest::newRow("qrc:/testdata") << u":/testdata/"_s << QDirIterator::IteratorFlags{}
                                << QDir::Filters(QDir::NoFilter) << QStringList(QLatin1String("*"))
                                << QString::fromLatin1(":/testdata/entrylist").split(QLatin1String(","));
-    QTest::newRow(":/testdata/entrylist") << QString::fromLatin1(":/testdata/entrylist") << QDirIterator::IteratorFlags{}
+    QTest::newRow("qrc:/testdata/entrylist") << u":/testdata/entrylist"_s << QDirIterator::IteratorFlags{}
                                << QDir::Filters(QDir::NoFilter) << QStringList(QLatin1String("*"))
                                << QString::fromLatin1(":/testdata/entrylist/directory,:/testdata/entrylist/file").split(QLatin1String(","));
-    QTest::newRow(":/testdata recursive") << QString::fromLatin1(":/testdata") << QDirIterator::IteratorFlags(QDirIterator::Subdirectories)
+    QTest::newRow("qrc:/testdata recursive") << u":/testdata"_s
+                                         << QDirIterator::IteratorFlags(QDirIterator::Subdirectories)
                                          << QDir::Filters(QDir::NoFilter) << QStringList(QLatin1String("*"))
                                          << QString::fromLatin1(":/testdata/entrylist,:/testdata/entrylist/directory,:/testdata/entrylist/directory/dummy,:/testdata/entrylist/file").split(QLatin1String(","));
 }
@@ -407,8 +390,8 @@ void tst_QDirIterator::iterateResource()
     sortedEntries.sort();
 
     if (sortedEntries != list) {
-        qDebug() << "EXPECTED:" << sortedEntries;
         qDebug() << "ACTUAL:" << list;
+        qDebug() << "EXPECTED:" << sortedEntries;
     }
 
     QCOMPARE(list, sortedEntries);
@@ -455,16 +438,20 @@ public:
         : QFSFileEngine(fileName)
     { }
 
-    QAbstractFileEngineIterator *beginEntryList(QDir::Filters, const QStringList &) override
-    { return 0; }
+    IteratorUniquePtr
+    beginEntryList(const QString &, QDirListing::IteratorFlags, const QStringList &) override
+    { return nullptr; }
 };
 
 class EngineWithNoIteratorHandler : public QAbstractFileEngineHandler
 {
+    Q_DISABLE_COPY_MOVE(EngineWithNoIteratorHandler)
 public:
-    QAbstractFileEngine *create(const QString &fileName) const override
+    EngineWithNoIteratorHandler() = default;
+
+    std::unique_ptr<QAbstractFileEngine> create(const QString &fileName) const override
     {
-        return new EngineWithNoIterator(fileName);
+        return std::make_unique<EngineWithNoIterator>(fileName);
     }
 };
 #endif
@@ -477,6 +464,41 @@ void tst_QDirIterator::engineWithNoIterator()
     QDir("entrylist").entryList();
     QVERIFY(true); // test that the above line doesn't crash
 }
+
+class CustomEngineHandler : public QAbstractFileEngineHandler
+{
+    Q_DISABLE_COPY_MOVE(CustomEngineHandler)
+public:
+    CustomEngineHandler() = default;
+
+    std::unique_ptr<QAbstractFileEngine> create(const QString &fileName) const override
+    {
+        // We want to test QFSFileEngine specifically, so force QDirIterator to use it
+        // over the default QFileSystemEngine
+        return std::make_unique<QFSFileEngine>(fileName);
+    }
+};
+
+void tst_QDirIterator::testQFsFileEngineIterator()
+{
+    QFETCH(QString, dirName);
+    QFETCH(QStringList, nameFilters);
+    QFETCH(QDir::Filters, filters);
+    QFETCH(QDirIterator::IteratorFlags, flags);
+
+    if (dirName == u"empty")
+        return; // This row isn't useful in this test
+
+    CustomEngineHandler handler;
+    bool isEmpty = true;
+    QDirIterator iter(dirName, nameFilters, filters, flags);
+    while (iter.hasNext()) {
+        const QFileInfo &fi = iter.nextFileInfo();
+        if (fi.filePath().contains(u"entrylist"))
+            isEmpty = false;  // At least one entry in `entrylist` dir
+    }
+    QVERIFY(!isEmpty);
+}
 #endif
 
 void tst_QDirIterator::absoluteFilePathsFromRelativeIteratorPath()
@@ -519,8 +541,15 @@ void tst_QDirIterator::longPath()
     while (dir.exists(dirName) || dir.mkdir(dirName)) {
         ++n;
         dirName.append('x');
+        if (n >= 20480)
+        {
+            break;
+        }
+    }
+    if (n >= 20480)
+    {
+        qWarning("No maximum length on directory names");
     }
-
     QDirIterator it(dir.absolutePath(), QDir::NoDotAndDotDot|QDir::Dirs, QDirIterator::Subdirectories);
     int m = 0;
     while (it.hasNext()) {
@@ -529,13 +558,14 @@ void tst_QDirIterator::longPath()
     }
 
     QCOMPARE(n, m);
-
     dirName.chop(1);
-    while (dirName.length() > 0 && dir.exists(dirName) && dir.rmdir(dirName)) {
+    while (dirName.size() > 0 && dir.exists(dirName) && dir.rmdir(dirName)) {
+        --n;
         dirName.chop(1);
     }
-    dir.cdUp();
-    dir.rmdir("longpaths");
+    QCOMPARE(n, 0);
+    QVERIFY(dir.cdUp());
+    QVERIFY(dir.rmdir("longpaths"));
 }
 
 void tst_QDirIterator::dirorder()
@@ -614,6 +644,23 @@ void tst_QDirIterator::hiddenDirs_hiddenFiles()
 }
 #endif // Q_OS_WIN
 
+void tst_QDirIterator::hasNextFalseNoCrash()
+{
+    QVERIFY(QFileInfo(u"empty"_s).exists());
+    QDirIterator iter(u"empty"_s);
+    int count = 0;
+    while (iter.hasNext()) {
+        iter.next();
+        ++count;
+    }
+    QVERIFY(count > 0);
+    QVERIFY(!iter.hasNext());
+    // QTBUG-130142
+    // When the iteration reaches the end, calling next() returns an empty string
+    // and no crash happens
+    QVERIFY(iter.next().isEmpty());
+}
+
 QTEST_MAIN(tst_QDirIterator)
 
 #include "tst_qdiriterator.moc"
