@@ -2343,7 +2343,7 @@ void tst_QMatrixNxN::rotate4x4()
     ROTATE4(2.0f, 3.0f, 0.0f, 1.0f, p1x, p1y, p1z, p1w);
     p1x /= p1w;
     p1y /= p1w;
-    p1z /= p1w;
+    Q_UNUSED(p1z);
 
     QVector3D v1(2.0f, 3.0f, -4.0f);
     QVector3D v2 = m1.map(v1);
