@@ -1,30 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the test suite of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #include "../../../shared/highdpi.h"
 
@@ -47,6 +22,8 @@
 #include <qpa/qplatformtheme.h>
 #include <qpa/qplatformtheme_p.h>
 
+#include <QtWidgets/private/qapplication_p.h>
+
 QT_FORWARD_DECLARE_CLASS(QDialog)
 
 // work around function being protected
@@ -70,6 +47,8 @@ private slots:
     void showMinimized();
     void showFullScreen();
     void showAsTool();
+    void showWithoutActivating_data();
+    void showWithoutActivating();
     void toolDialogPosition();
     void deleteMainDefault();
     void deleteInExec();
@@ -86,6 +65,9 @@ private slots:
     void virtualsOnClose();
     void deleteOnDone();
     void quitOnDone();
+    void focusWidgetAfterOpen();
+
+    void closeParentOfVisibleDialog();
 };
 
 // Testing get/set functions
@@ -155,7 +137,7 @@ void tst_QDialog::defaultButtons()
     pushThree->setAutoDefault(false);
 
     testWidget.show();
-    QApplication::setActiveWindow(&testWidget);
+    QApplicationPrivate::setActiveWindow(&testWidget);
     QVERIFY(QTest::qWaitForWindowExposed(&testWidget));
 
     push->setDefault(true);
@@ -304,9 +286,12 @@ void tst_QDialog::showFullScreen()
 
 void tst_QDialog::showAsTool()
 {
-#if defined(Q_OS_UNIX)
-    QSKIP("Qt/X11: Skipped since activeWindow() is not respected by all window managers");
-#endif
+    if (QStringList{"xcb", "offscreen"}.contains(QGuiApplication::platformName()))
+        QSKIP("activeWindow() is not respected by all Xcb window managers and the offscreen plugin");
+
+    if (!QGuiApplicationPrivate::platformIntegration()->hasCapability(QPlatformIntegration::WindowActivation))
+        QSKIP("QWindow::requestActivate() is not supported.");
+
     DummyDialog testWidget;
     testWidget.resize(200, 200);
     testWidget.setWindowTitle(QTest::currentTestFunction());
@@ -322,6 +307,33 @@ void tst_QDialog::showAsTool()
     }
 }
 
+void tst_QDialog::showWithoutActivating_data()
+{
+    QTest::addColumn<bool>("showWithoutActivating");
+    QTest::addColumn<int>("focusInCount");
+
+    QTest::addRow("showWithoutActivating") << true << 0;
+    QTest::addRow("showWithActivating") << false << 1;
+}
+
+void tst_QDialog::showWithoutActivating()
+{
+    QFETCH(bool, showWithoutActivating);
+    QFETCH(int, focusInCount);
+
+    struct Dialog : public QDialog
+    {
+        int focusInCount = 0;
+    protected:
+        void focusInEvent(QFocusEvent *) override { ++focusInCount; }
+    } dialog;
+    dialog.setAttribute(Qt::WA_ShowWithoutActivating, showWithoutActivating);
+
+    dialog.show();
+    QVERIFY(QTest::qWaitForWindowExposed(&dialog));
+    QCOMPARE(dialog.focusInCount, focusInCount);
+}
+
 // Verify that pos() returns the same before and after show()
 // for a dialog with the Tool window type.
 void tst_QDialog::toolDialogPosition()
@@ -474,6 +486,9 @@ void tst_QDialog::snapToDefaultButton()
 #else
     if (!QGuiApplication::platformName().compare(QLatin1String("wayland"), Qt::CaseInsensitive))
         QSKIP("This platform does not support setting the cursor position.");
+#ifdef Q_OS_ANDROID
+    QSKIP("Android does not support cursor");
+#endif
 
     const QRect dialogGeometry(QGuiApplication::primaryScreen()->availableGeometry().topLeft()
                                + QPoint(100, 100), QSize(200, 200));
@@ -615,7 +630,9 @@ void tst_QDialog::virtualsOnClose()
         dialog.show();
         QVERIFY(QTest::qWaitForWindowExposed(&dialog));
         dialog.accept();
-        QCOMPARE(dialog.closeEventCount, 0); // we only hide the dialog
+        // we used to only hide the dialog, and we still don't want a
+        // closeEvent call for application-triggered calls to QDialog::done
+        QCOMPARE(dialog.closeEventCount, 0);
         QCOMPARE(dialog.acceptCount, 1);
         QCOMPARE(dialog.rejectCount, 0);
         QCOMPARE(dialog.doneCount, 1);
@@ -626,7 +643,7 @@ void tst_QDialog::virtualsOnClose()
         dialog.show();
         QVERIFY(QTest::qWaitForWindowExposed(&dialog));
         dialog.reject();
-        QCOMPARE(dialog.closeEventCount, 0); // we only hide the dialog
+        QCOMPARE(dialog.closeEventCount, 0);
         QCOMPARE(dialog.acceptCount, 0);
         QCOMPARE(dialog.rejectCount, 1);
         QCOMPARE(dialog.doneCount, 1);
@@ -680,7 +697,7 @@ void tst_QDialog::virtualsOnClose()
         // Qt doesn't deliver events to QWidgets closed during destruction
         QCOMPARE(filter.closeEventCount, 0);
         // QDialog doesn't emit signals when closed by destruction
-        QCOMPARE(rejectedSpy.count(), 0);
+        QCOMPARE(rejectedSpy.size(), 0);
     }
 }
 
@@ -733,7 +750,42 @@ void tst_QDialog::quitOnDone()
     // also quit with a timer in case the test fails
     QTimer::singleShot(1000, QApplication::instance(), &QApplication::quit);
     QApplication::exec();
-    QCOMPARE(quitSpy.count(), 1);
+    QCOMPARE(quitSpy.size(), 1);
+}
+
+void tst_QDialog::focusWidgetAfterOpen()
+{
+    QDialog dialog;
+    dialog.setLayout(new QVBoxLayout);
+
+    QPushButton *pb1 = new QPushButton;
+    QPushButton *pb2 = new QPushButton;
+    dialog.layout()->addWidget(pb1);
+    dialog.layout()->addWidget(pb2);
+
+    pb2->setFocus();
+    QCOMPARE(dialog.focusWidget(), static_cast<QWidget *>(pb2));
+
+    dialog.open();
+    QCOMPARE(dialog.focusWidget(), static_cast<QWidget *>(pb2));
+}
+
+void tst_QDialog::closeParentOfVisibleDialog()
+{
+    QWidget widget;
+    widget.show();
+
+    // On macOS, this dialog becomes a sheet and triggered an assert when
+    // the widget it lived in was already gone by the time the sheet
+    // got closed. QTBUG-128302
+    QDialog dialog(&widget);
+    dialog.open();
+
+    QTimer::singleShot(0, &widget, [&]{
+        widget.close();
+    });
+
+    QTRY_VERIFY(!widget.isVisible());
 }
 
 QTEST_MAIN(tst_QDialog)
