@@ -117,6 +117,11 @@ private slots:
 
     void nestedTearOffDetached();
     void closeMenuOnClickIfMouseHasntMoved();
+#if QT_CONFIG(shortcut) && !defined(Q_OS_DARWIN)
+    void dontSelectDisabledActionByShortcut();
+#endif
+
+    void invisibleActions();
 
 protected slots:
     void onActivated(QAction*);
@@ -1592,12 +1597,12 @@ void tst_QMenu::transientParent()
     window.menuBar()->setNativeMenuBar(false);
     centerOnScreen(&window);
 
-    QMenu *fileMenu = new QMenu("&File");
-    QAction *exitAct = new QAction("Exit");
+    QMenu *fileMenu = new QMenu("&File", &window);
+    QAction *exitAct = new QAction("Exit", &window);
     fileMenu->addAction(exitAct);
 
-    QMenu *editMenu = new QMenu("&Edit");
-    QAction *undoAct = new QAction("Undo");
+    QMenu *editMenu = new QMenu("&Edit", &window);
+    QAction *undoAct = new QAction("Undo", &window);
     editMenu->addAction(undoAct);
 
     QMenuBar *menuBar = new QMenuBar;
@@ -1977,11 +1982,13 @@ void tst_QMenu::QTBUG_61039_menu_shortcuts()
     if (!QGuiApplicationPrivate::platformIntegration()->hasCapability(QPlatformIntegration::WindowActivation))
         QSKIP("Window activation is not supported");
 
-    QAction *actionKamen = new QAction("Action Kamen");
+    QObject reaper;
+
+    QAction *actionKamen = new QAction("Action Kamen", &reaper);
 #if QT_CONFIG(shortcut)
     actionKamen->setShortcut(QKeySequence(QLatin1String("K")));
 #endif
-    QAction *actionJoe = new QAction("Action Joe");
+    QAction *actionJoe = new QAction("Action Joe", &reaper);
 #if QT_CONFIG(shortcut)
     actionJoe->setShortcut(QKeySequence(QLatin1String("Ctrl+J")));
 #endif
@@ -2145,5 +2152,64 @@ void tst_QMenu::closeMenuOnClickIfMouseHasntMoved()
     QTest::mouseClick(&contextMenu, Qt::RightButton, {}, contextMenu.mapFromGlobal(pos));
 }
 
+void tst_QMenu::invisibleActions()
+{
+    QWidget window;
+    window.resize(100, 100);
+    window.show();
+
+    const QPoint globalPos = window.mapToGlobal(window.rect().center());
+
+    QVERIFY(QTest::qWaitForWindowExposed(&window));
+
+    QMenu contextMenu;
+    QList<QAction *> actions;
+    for (int i = 0; i < 5; ++i)
+        actions << contextMenu.addAction("action");
+    QVERIFY(contextMenu.sizeHint().isValid());
+
+    contextMenu.popup(globalPos);
+    QVERIFY(contextMenu.isVisible());
+
+    contextMenu.close();
+
+    for (const auto &action : actions)
+        action->setVisible(false);
+
+    contextMenu.popup(globalPos);
+    QVERIFY(!contextMenu.isVisible());
+    QVERIFY(!contextMenu.exec());
+
+    // a QMenu might not have any (visible) actions, but still have contents
+    QPushButton *buttonInMenu = new QPushButton(&contextMenu);
+    buttonInMenu->show();
+
+    contextMenu.popup(globalPos);
+    QVERIFY(contextMenu.isVisible());
+}
+
+#if QT_CONFIG(shortcut) && !defined(Q_OS_DARWIN)
+void tst_QMenu::dontSelectDisabledActionByShortcut()
+{
+    if (!QGuiApplicationPrivate::platformIntegration()->hasCapability(QPlatformIntegration::WindowActivation))
+        QSKIP("Window activation is not supported");
+
+    QMainWindow w;
+    auto mb = w.menuBar();
+    auto m = mb->addMenu("me&nu");
+    auto zero = m->addAction("placeholder");
+    auto first = m->addAction("disabled &o");
+    auto second = m->addAction(QStringLiteral("enabled &o"));
+
+    QSignalSpy spy(second, &QAction::triggered);
+    first->setDisabled(true);
+    w.show();
+    QVERIFY(QTest::qWaitForWindowActive(&w));
+    QTest::keyClick(&w, Qt::Key_N, Qt::AltModifier);
+    QTest::keyClick(m, Qt::Key_O, Qt::NoModifier);
+    QCOMPARE(spy.count(), 1);
+}
+#endif
+
 QTEST_MAIN(tst_QMenu)
 #include "tst_qmenu.moc"
